@@ -16,9 +16,6 @@ import (
 	"github.com/spf13/cobra"
 	"gopkg.in/yaml.v3"
 
-	"github.com/open-policy-agent/opa/ast"
-	"github.com/open-policy-agent/opa/format"
-
 	"github.com/styrainc/regal/internal/git"
 	rio "github.com/styrainc/regal/internal/io"
 	"github.com/styrainc/regal/internal/util"
@@ -27,6 +24,8 @@ import (
 	"github.com/styrainc/regal/pkg/fixer/fileprovider"
 	"github.com/styrainc/regal/pkg/fixer/fixes"
 	"github.com/styrainc/regal/pkg/linter"
+
+	rutil "github.com/styrainc/roast/pkg/util"
 )
 
 // fixCommandParams is similar to the lint params, but with some fields such as profiling removed.
@@ -204,6 +203,8 @@ func fix(args []string, params *fixCommandParams) error {
 	} else {
 		regalDir, err = config.FindRegalDirectory(configSearchPath)
 		if err == nil {
+			defer regalDir.Close()
+
 			customRulesPath := filepath.Join(regalDir.Name(), rio.PathSeparator, "rules")
 			if _, err = os.Stat(customRulesPath); err == nil {
 				customRulesDir = customRulesPath
@@ -232,9 +233,13 @@ func fix(args []string, params *fixCommandParams) error {
 		l = l.WithIgnore(params.ignoreFiles.v)
 	}
 
+	if regalDir != nil {
+		l = l.WithPathPrefix(filepath.Dir(regalDir.Name()))
+	}
+
 	var userConfig config.Config
 
-	userConfigFile, err := readUserConfig(params, regalDir)
+	userConfigFile, err := readUserConfig(params, configSearchPath)
 
 	switch {
 	case err == nil:
@@ -270,14 +275,15 @@ func fix(args []string, params *fixCommandParams) error {
 	f := fixer.NewFixer()
 	f.RegisterRoots(roots...)
 	f.RegisterFixes(fixes.NewDefaultFixes()...)
-	f.RegisterMandatoryFixes(
-		&fixes.Fmt{
-			NameOverride: "use-rego-v1",
-			OPAFmtOpts: format.Opts{
-				RegoVersion: ast.RegoV0CompatV1,
-			},
-		},
-	)
+
+	if userConfigFile != nil {
+		versionsMap, err := config.AllRegoVersions(filepath.Dir(userConfigFile.Name()), &userConfig)
+		if err != nil {
+			return fmt.Errorf("failed to get all Rego versions: %w", err)
+		}
+
+		f.SetRegoVersionsMap(versionsMap)
+	}
 
 	if !slices.Contains([]string{"error", "rename"}, params.conflictMode) {
 		return fmt.Errorf("invalid conflict mode: %s, expected 'error' or 'rename'", params.conflictMode)
@@ -345,35 +351,28 @@ func fix(args []string, params *fixCommandParams) error {
 		return errors.New("fixing failed due to conflicts")
 	}
 
-	gitRepo, err := git.FindGitRepo(args...)
-	if err != nil {
-		return fmt.Errorf("failed to establish git repo: %w", err)
-	}
-
-	if gitRepo == "" && !params.dryRun && !params.force {
-		return errors.New("no git repo found to support undo, use --force to override")
-	}
-
-	// if the fixer is being run in a git repo, we must not fix files that have
-	// been changed.
 	if !params.dryRun && !params.force {
-		changedFiles := make(map[string]struct{})
-
-		if gitRepo != "" {
-			cf, err := git.GetChangedFiles(gitRepo)
-			if err != nil {
-				return fmt.Errorf("failed to get changed files: %w", err)
-			}
-
-			for _, f := range cf {
-				changedFiles[f] = struct{}{}
-			}
+		gitRepo, err := git.FindGitRepo(args...)
+		if err != nil {
+			return fmt.Errorf("failed to establish git repo (use --force to override): %w", err)
 		}
 
+		if gitRepo == "" {
+			return errors.New("no git repo found to support undo (use --force to override)")
+		}
+
+		// if the fixer is being run in a git repo, we must not fix files that have changes
+		cf, err := git.GetChangedFiles(gitRepo)
+		if err != nil {
+			return fmt.Errorf("failed to get changed files: %w", err)
+		}
+
+		changedFiles := rutil.NewSet(cf...)
+
 		var conflictingFiles []string
 
 		for _, file := range fileProvider.ModifiedFiles() {
-			if _, ok := changedFiles[file]; ok {
+			if changedFiles.Contains(file) {
 				conflictingFiles = append(conflictingFiles, file)
 			}
 		}
@@ -382,7 +381,7 @@ func fix(args []string, params *fixCommandParams) error {
 			return fmt.Errorf(
 				`the following files have been changed since the fixer was run:
 - %s
-please run fix from a clean state to support the use of git checkout for undo`,
+please run fix from a clean state to support the use of git to undo, or use --force to ignore`,
 				strings.Join(conflictingFiles, "\n- "),
 			)
 		}
@@ -401,7 +400,7 @@ please run fix from a clean state to support the use of git checkout for undo`,
 				return fmt.Errorf("failed to get file %s: %w", file, err)
 			}
 
-			fmt.Fprintln(outputWriter, string(fc))
+			fmt.Fprintln(outputWriter, fc)
 			fmt.Fprintln(outputWriter, "----------")
 		}
 
@@ -446,7 +445,7 @@ please run fix from a clean state to support the use of git checkout for undo`,
 				return fmt.Errorf("failed to create directory for file %s: %w", file, err)
 			}
 
-			if err = os.WriteFile(file, fc, fileMode); err != nil {
+			if err = os.WriteFile(file, []byte(fc), fileMode); err != nil {
 				return fmt.Errorf("failed to write file %s: %w", file, err)
 			}
 		}
