@@ -9,35 +9,37 @@ import (
 	"errors"
 	"fmt"
 	"os"
+	"slices"
 	"strings"
 	"time"
 
 	"github.com/spf13/cobra"
 	"github.com/spf13/pflag"
 
-	"github.com/open-policy-agent/opa/ast"
-	"github.com/open-policy-agent/opa/bundle"
-	"github.com/open-policy-agent/opa/cover"
-	"github.com/open-policy-agent/opa/storage"
-	"github.com/open-policy-agent/opa/storage/inmem"
-	"github.com/open-policy-agent/opa/tester"
-	"github.com/open-policy-agent/opa/topdown"
-	"github.com/open-policy-agent/opa/util"
-	"github.com/open-policy-agent/opa/version"
+	"github.com/open-policy-agent/opa/v1/ast"
+	"github.com/open-policy-agent/opa/v1/bundle"
+	"github.com/open-policy-agent/opa/v1/cover"
+	"github.com/open-policy-agent/opa/v1/loader"
+	"github.com/open-policy-agent/opa/v1/storage"
+	"github.com/open-policy-agent/opa/v1/storage/inmem"
+	"github.com/open-policy-agent/opa/v1/tester"
+	"github.com/open-policy-agent/opa/v1/topdown"
+	outil "github.com/open-policy-agent/opa/v1/util"
+	"github.com/open-policy-agent/opa/v1/version"
 
 	rbundle "github.com/styrainc/regal/bundle"
 	"github.com/styrainc/regal/internal/compile"
 	rio "github.com/styrainc/regal/internal/io"
 	"github.com/styrainc/regal/pkg/builtins"
 	"github.com/styrainc/regal/pkg/config"
+
+	rutil "github.com/styrainc/roast/pkg/util"
 )
 
-const (
-	benchmarkGoBenchOutput = "gobench"
-)
+const benchmarkGoBenchOutput = "gobench"
 
 type testCommandParams struct {
-	outputFormat *util.EnumFlag
+	outputFormat *outil.EnumFlag
 	runRegex     string
 	ignore       []string
 	threshold    float64
@@ -52,9 +54,13 @@ type testCommandParams struct {
 	varValues    bool
 }
 
+type loaderFilter struct {
+	Ignore []string
+}
+
 func newTestCommandParams() *testCommandParams {
 	return &testCommandParams{
-		outputFormat: util.NewEnumFlag(formatPretty, []string{
+		outputFormat: outil.NewEnumFlag(formatPretty, []string{
 			formatPretty,
 			formatJSON,
 			benchmarkGoBenchOutput,
@@ -62,6 +68,12 @@ func newTestCommandParams() *testCommandParams {
 	}
 }
 
+func (f loaderFilter) Apply(abspath string, info os.FileInfo, depth int) bool {
+	return slices.ContainsFunc(f.Ignore, func(s string) bool {
+		return loader.GlobExcludeName(s, 1)(abspath, info, depth)
+	})
+}
+
 var testParams = newTestCommandParams() //nolint: gochecknoglobals
 
 var testCommand = &cobra.Command{ //nolint: gochecknoglobals
@@ -218,21 +230,20 @@ func opaTest(args []string) int {
 func moduleLoader(regalRules *bundle.Bundle) ast.ModuleLoader {
 	// We use the package declarations to know which modules we still need, and return
 	// those from the embedded regal bundle.
-	extra := make(map[string]struct{}, len(regalRules.Modules))
-
+	extra := rutil.NewSet[string]()
 	for _, mod := range regalRules.Modules {
-		extra[mod.Parsed.Package.Path.String()] = struct{}{}
+		extra.Add(mod.Parsed.Package.Path.String())
 	}
 
 	return func(present map[string]*ast.Module) (map[string]*ast.Module, error) {
 		for _, mod := range present {
-			delete(extra, mod.Package.Path.String())
+			extra.Remove(mod.Package.Path.String())
 		}
 
 		extraMods := map[string]*ast.Module{}
 
 		for id, mod := range regalRules.ParsedModules("bundle") {
-			if _, ok := extra[mod.Package.Path.String()]; ok {
+			if extra.Contains(mod.Package.Path.String()) {
 				extraMods[id] = mod
 			}
 		}
