@@ -20,26 +20,23 @@ import (
 	"github.com/google/go-cmp/cmp"
 	"gopkg.in/yaml.v3"
 
-	"github.com/open-policy-agent/opa/tester"
+	"github.com/open-policy-agent/opa/v1/tester"
 
 	"github.com/styrainc/regal/internal/testutil"
 	"github.com/styrainc/regal/pkg/config"
 	"github.com/styrainc/regal/pkg/report"
+
+	rutil "github.com/styrainc/roast/pkg/util"
 )
 
 func readProvidedConfig(t *testing.T) config.Config {
 	t.Helper()
 
 	cwd := testutil.Must(os.Getwd())(t)
-
-	configPath := filepath.Join(cwd, "..", "bundle", "regal", "config", "provided", "data.yaml")
-	bs, err := os.ReadFile(configPath)
-	if err != nil {
-		t.Fatalf("failed to read config from %q: %v", configPath, err)
-	}
+	bs := testutil.MustReadFile(t, filepath.Join(cwd, "..", "bundle", "regal", "config", "provided", "data.yaml"))
 
 	var cfg config.Config
-	if err = yaml.Unmarshal(bs, &cfg); err != nil {
+	if err := yaml.Unmarshal(bs, &cfg); err != nil {
 		t.Fatalf("failed to unmarshal config: %v", err)
 	}
 
@@ -139,6 +136,9 @@ func TestLintNonExistentDir(t *testing.T) {
 	}
 }
 
+// NOTE(ae): this test *will fail* when run via "go test -e2e" and a normal development binary is used for the test.
+// This is because proposing fixes is only enabled when regal is built with the "regal_standalone" build tag. I spent
+// way more time on chasing that down than I would have wanted, so leaving this here for future me and others.
 func TestLintProposeToRunFix(t *testing.T) {
 	t.Parallel()
 	stdout, stderr := bytes.Buffer{}, bytes.Buffer{}
@@ -146,30 +146,43 @@ func TestLintProposeToRunFix(t *testing.T) {
 	cwd := testutil.Must(os.Getwd())(t)
 
 	// using a test rego file that only yields a few violations
-	err := regal(&stdout, &stderr)("lint", cwd+filepath.FromSlash("/testdata/violations/rule_named_if.rego"))
+	err := regal(&stdout, &stderr)(
+		"lint",
+		"--config-file", filepath.Join(cwd, "e2e_conf.yaml"),
+		cwd+filepath.FromSlash("/testdata/v0/rule_named_if.rego"))
 
 	expectExitCode(t, err, 3, &stdout, &stderr)
 
 	if exp, act := "", stderr.String(); exp != act {
-		t.Errorf("expected stderr %q, got %q", exp, act)
+		t.Fatalf("expected stderr %q, got %q", exp, act)
 	}
 
 	act := strings.Split(stdout.String(), "\n")
 	act = act[len(act)-5:]
-	exp := []string{"1 file linted. 5 violations found.", "", "Hint: 2/5 violations can be automatically fixed (directory-package-mismatch, use-rego-v1)", "      Run regal fix --help for more details.", ""}
+	exp := []string{
+		"1 file linted. 2 violations found.",
+		"",
+		"Hint: 2/2 violations can be automatically fixed (directory-package-mismatch, opa-fmt)",
+		"      Run regal fix --help for more details.",
+		"",
+	}
 	if diff := cmp.Diff(act, exp); diff != "" {
 		t.Errorf("unexpected stdout trailer: (-want, +got):\n%s", diff)
 	}
 }
 
-func TestLintAllViolations(t *testing.T) {
+func TestLintV1Violations(t *testing.T) {
 	t.Parallel()
 	stdout, stderr := bytes.Buffer{}, bytes.Buffer{}
 
 	cwd := testutil.Must(os.Getwd())(t)
-	cfg := readProvidedConfig(t)
 
-	err := regal(&stdout, &stderr)("lint", "--format", "json", cwd+filepath.FromSlash("/testdata/violations"))
+	err := regal(&stdout, &stderr)(
+		"lint",
+		"--format", "json",
+		"--config-file", filepath.Join(cwd, "e2e_conf.yaml"),
+		cwd+filepath.FromSlash("/testdata/violations"),
+	)
 
 	expectExitCode(t, err, 3, &stdout, &stderr)
 
@@ -182,41 +195,45 @@ func TestLintAllViolations(t *testing.T) {
 		t.Fatalf("expected JSON response, got %v", stdout.String())
 	}
 
-	ruleNames := make(map[string]struct{})
+	ruleNames := rutil.NewSet[string]()
+	excludedRules := rutil.NewSet(
+		"implicit-future-keywords",
+		"use-if",
+		"use-contains",
+		"internal-entrypoint",
+		"file-length",
+		"rule-named-if",
+		"use-rego-v1",
+		"deprecated-builtin",
+		"import-shadows-import",
+	)
 
-	excludedRules := map[string]struct{}{
-		"implicit-future-keywords": {},
-		"use-if":                   {},
-		"use-contains":             {},
-		"internal-entrypoint":      {},
-		"file-length":              {},
-	}
+	cfg := readProvidedConfig(t)
 
 	for _, category := range cfg.Rules {
 		for ruleName, rule := range category {
-			if _, isExcluded := excludedRules[ruleName]; !isExcluded && rule.Level != "ignore" {
-				ruleNames[ruleName] = struct{}{}
+			if !excludedRules.Contains(ruleName) && rule.Level != "ignore" {
+				ruleNames.Add(ruleName)
 			}
 		}
 	}
 
 	// Note that some violations occur more than one time.
-	violationNames := make(map[string]struct{})
-
+	violationNames := rutil.NewSet[string]()
 	for _, violation := range rep.Violations {
-		violationNames[violation.Title] = struct{}{}
+		violationNames.Add(violation.Title)
 	}
 
-	if len(ruleNames) != len(violationNames) {
-		for ruleName := range ruleNames {
-			if _, ok := violationNames[ruleName]; !ok {
+	if ruleNames.Size() != violationNames.Size() {
+		for _, ruleName := range ruleNames.Items() {
+			if !violationNames.Contains(ruleName) {
 				t.Errorf("expected violation for rule %q", ruleName)
 			}
 		}
 	}
 }
 
-func TestLintNotRegoV1Violations(t *testing.T) {
+func TestLintV0NoRegoV1ImportViolations(t *testing.T) {
 	t.Parallel()
 
 	stdout, stderr := bytes.Buffer{}, bytes.Buffer{}
@@ -224,8 +241,8 @@ func TestLintNotRegoV1Violations(t *testing.T) {
 	cwd := testutil.Must(os.Getwd())(t)
 
 	err := regal(&stdout, &stderr)("lint", "--format", "json", "--config-file",
-		cwd+filepath.FromSlash("/testdata/configs/not_rego_v1.yaml"),
-		cwd+filepath.FromSlash("/testdata/not_rego_v1"))
+		cwd+filepath.FromSlash("/testdata/configs/v0.yaml"),
+		cwd+filepath.FromSlash("/testdata/v0/"))
 
 	expectExitCode(t, err, 3, &stdout, &stderr)
 
@@ -238,25 +255,52 @@ func TestLintNotRegoV1Violations(t *testing.T) {
 		t.Fatalf("expected JSON response, got %v", stdout.String())
 	}
 
-	expected := map[string]struct{}{
-		"implicit-future-keywords": {},
-		"use-if":                   {},
-		"use-contains":             {},
+	// Note that some violations occur more than one time.
+	violationNames := rutil.NewSet[string]()
+	expected := rutil.NewSet("implicit-future-keywords", "use-if", "use-contains")
+
+	for _, violation := range rep.Violations {
+		violationNames.Add(violation.Title)
+	}
+
+	if diff := expected.Diff(violationNames); diff.Size() > 0 {
+		t.Errorf("expected violations for rules %v, got %v", expected.Items(), violationNames.Items())
+	}
+}
+
+func TestLintV0WithRegoV1ImportViolations(t *testing.T) {
+	t.Parallel()
+
+	stdout, stderr := bytes.Buffer{}, bytes.Buffer{}
+
+	cwd := testutil.Must(os.Getwd())(t)
+
+	err := regal(&stdout, &stderr)(
+		"lint", "--format", "json",
+		"--config-file", cwd+filepath.FromSlash("/testdata/configs/v0-with-import-rego-v1.yaml"),
+		cwd+filepath.FromSlash("/testdata/v0/"))
+
+	expectExitCode(t, err, 3, &stdout, &stderr)
+
+	if exp, act := "", stderr.String(); exp != act {
+		t.Errorf("expected stderr:\n%s\ngot\n%s", exp, act)
+	}
+
+	var rep report.Report
+	if err = json.Unmarshal(stdout.Bytes(), &rep); err != nil {
+		t.Fatalf("expected JSON response, got %v", stdout.String())
 	}
 
 	// Note that some violations occur more than one time.
-	violationNames := make(map[string]struct{})
+	violationNames := rutil.NewSet[string]()
+	expected := rutil.NewSet("use-if", "use-contains", "use-rego-v1", "rule-named-if")
 
 	for _, violation := range rep.Violations {
-		violationNames[violation.Title] = struct{}{}
+		violationNames.Add(violation.Title)
 	}
 
-	if len(expected) != len(violationNames) {
-		for ruleName := range expected {
-			if _, ok := violationNames[ruleName]; !ok {
-				t.Errorf("expected violation for rule %q", ruleName)
-			}
-		}
+	if diff := expected.Diff(violationNames); diff.Size() > 0 {
+		t.Errorf("expected violations for rules %v, got %v", expected.Items(), violationNames.Items())
 	}
 }
 
@@ -308,13 +352,13 @@ func TestLintRuleIgnoreFiles(t *testing.T) {
 		t.Fatalf("expected JSON response, got %v", stdout.String())
 	}
 
-	violationNames := make(map[string]struct{})
+	violationNames := rutil.NewSet[string]()
 
 	for _, violation := range rep.Violations {
-		violationNames[violation.Title] = struct{}{}
+		violationNames.Add(violation.Title)
 	}
 
-	if _, ok := violationNames["prefer-snake-case"]; ok {
+	if violationNames.Contains("prefer-snake-case") {
 		t.Errorf("did not expect violation for rule %q as it is ignored", "prefer-snake-case")
 	}
 }
@@ -381,7 +425,7 @@ func TestAggregatesAreCollectedAndUsed(t *testing.T) {
 	t.Run("two policies — no violations expected", func(t *testing.T) {
 		stdout, stderr := bytes.Buffer{}, bytes.Buffer{}
 
-		err := regal(&stdout, &stderr)("lint", "--format", "json", "--rules",
+		err := regal(&stdout, &stderr)("lint", "--format", "json",
 			basedir+filepath.FromSlash("/custom/regal/rules/testcase/aggregates/custom_rules_using_aggregates.rego"),
 			basedir+filepath.FromSlash("/two_policies"))
 
@@ -409,7 +453,9 @@ func TestAggregatesAreCollectedAndUsed(t *testing.T) {
 	t.Run("three policies - violation expected", func(t *testing.T) {
 		stdout, stderr := bytes.Buffer{}, bytes.Buffer{}
 
-		err := regal(&stdout, &stderr)("lint", "--format", "json", "--rules",
+		err := regal(&stdout, &stderr)("lint", "--format", "json",
+			"--config-file", filepath.Join(cwd, "e2e_conf.yaml"),
+			"--rules",
 			basedir+filepath.FromSlash("/custom/regal/rules/testcase/aggregates/custom_rules_using_aggregates.rego"),
 			basedir+filepath.FromSlash("/three_policies"))
 
@@ -433,7 +479,9 @@ func TestAggregatesAreCollectedAndUsed(t *testing.T) {
 	t.Run("custom policy where nothing aggregate is a violation", func(t *testing.T) {
 		stdout, stderr := bytes.Buffer{}, bytes.Buffer{}
 
-		err := regal(&stdout, &stderr)("lint", "--format", "json", "--rules",
+		err := regal(&stdout, &stderr)("lint", "--format", "json",
+			"--config-file", filepath.Join(cwd, "e2e_conf.yaml"),
+			"--rules",
 			basedir+filepath.FromSlash("/custom/regal/rules/testcase/empty_aggregate/"),
 			basedir+filepath.FromSlash("/two_policies"))
 
@@ -461,8 +509,13 @@ func TestLintAggregateIgnoreDirective(t *testing.T) {
 	stdout, stderr := bytes.Buffer{}, bytes.Buffer{}
 	cwd := testutil.Must(os.Getwd())(t)
 
-	err := regal(&stdout, &stderr)("lint", "--format", "json",
-		cwd+filepath.FromSlash("/testdata/aggregates/ignore_directive"))
+	err := regal(&stdout, &stderr)(
+		"lint",
+		"--config-file", filepath.Join(cwd, "e2e_conf.yaml"),
+		"--format",
+		"json",
+		cwd+filepath.FromSlash("/testdata/aggregates/ignore_directive"),
+	)
 
 	expectExitCode(t, err, 3, &stdout, &stderr)
 
@@ -762,7 +815,13 @@ func TestLintPprof(t *testing.T) {
 		_ = os.Remove(pprofFile)
 	})
 
-	err := regal(&stdout, &stderr)("lint", "--pprof", "clock", cwd+filepath.FromSlash("/testdata/violations"))
+	err := regal(&stdout, &stderr)(
+		"lint",
+		// this overrides the ignore directives for e2e loaded from the config file
+		"--ignore-files=none",
+		"--pprof", "clock",
+		cwd+filepath.FromSlash("/testdata/violations"),
+	)
 
 	expectExitCode(t, err, 3, &stdout, &stderr)
 
@@ -775,14 +834,19 @@ func TestFix(t *testing.T) {
 	t.Parallel()
 
 	stdout, stderr := bytes.Buffer{}, bytes.Buffer{}
-	td := t.TempDir()
 
 	initialState := map[string]string{
-		".regal/config.yaml": "", // needed to find the root in the right place
+		".regal/config.yaml": `
+project:
+  rego-version: 1
+  roots:
+    - path: v0
+      rego-version: 0
+    - path: v1
+      rego-version: 1
+`,
 		"foo/main.rego": `package wow
 
-import rego.v1
-
 #comment
 
 allow if {
@@ -791,14 +855,12 @@ allow if {
 `,
 		"foo/main_test.rego": `package wow_test
 
-test_allow {
+test_allow if {
 	true
 }
 `,
 		"foo/foo.rego": `package foo
 
-import rego.v1
-
 # present and correct
 
 allow if {
@@ -806,45 +868,68 @@ allow if {
 }
 `,
 		"bar/main.rego": `package wow["foo-bar"].baz
-
-import rego.v1
 `,
 		"bar/main_test.rego": `package wow["foo-bar"].baz_test
-test_allow {
+test_allow if {
 	true
 }
+`,
+		"v0/main.rego": `package v0
+
+#comment
+allow { true }
+`,
+		"v1/main.rego": `package v1
+
+#comment
+allow if { true }
 `,
 		"unrelated.txt": `foobar`,
 	}
 
-	for file, content := range initialState {
-		mustWriteToFile(t, filepath.Join(td, file), content)
-	}
+	td := testutil.TempDirectoryOf(t, initialState)
 
 	// --force is required to make the changes when there is no git repo
-	err := regal(&stdout, &stderr)("fix", "--force", filepath.Join(td, "foo"), filepath.Join(td, "bar"))
+	err := regal(&stdout, &stderr)(
+		"fix",
+		"--force",
+		filepath.Join(td, "foo"),
+		filepath.Join(td, "bar"),
+		filepath.Join(td, "v0"),
+		filepath.Join(td, "v1"),
+	)
 
 	// 0 exit status is expected as all violations should have been fixed
 	expectExitCode(t, err, 0, &stdout, &stderr)
 
-	exp := fmt.Sprintf(`8 fixes applied:
-In project root: %s
+	exp := fmt.Sprintf(`12 fixes applied:
+In project root: %[1]s
 bar/main.rego -> wow/foo-bar/baz/main.rego:
 - directory-package-mismatch
 
 bar/main_test.rego -> wow/foo-bar/baz/main_test.rego:
 - directory-package-mismatch
-- use-rego-v1
+- opa-fmt
 
 foo/main.rego -> wow/main.rego:
 - directory-package-mismatch
-- use-rego-v1
 - no-whitespace-comment
+- opa-fmt
 
 foo/main_test.rego -> wow/main_test.rego:
 - directory-package-mismatch
-- use-rego-v1
+- opa-fmt
 
+
+In project root: %[1]s/v0
+main.rego:
+- use-rego-v1
+- no-whitespace-comment
+
+In project root: %[1]s/v1
+main.rego:
+- no-whitespace-comment
+- opa-fmt
 `, td)
 
 	if act := stdout.String(); exp != act {
@@ -858,8 +943,6 @@ foo/main_test.rego -> wow/main_test.rego:
 	expectedState := map[string]string{
 		"foo/foo.rego": `package foo
 
-import rego.v1
-
 # present and correct
 
 allow if {
@@ -867,28 +950,32 @@ allow if {
 }
 `,
 		"wow/foo-bar/baz/main.rego": `package wow["foo-bar"].baz
-
-import rego.v1
 `,
 		"wow/foo-bar/baz/main_test.rego": `package wow["foo-bar"].baz_test
 
-import rego.v1
-
 test_allow := true
 `,
 		"wow/main.rego": `package wow
 
-import rego.v1
-
 # comment
 
 allow := true
 `,
 		"wow/main_test.rego": `package wow_test
 
+test_allow := true
+`,
+		"v0/main.rego": `package v0
+
 import rego.v1
 
-test_allow := true
+# comment
+allow := true
+`,
+		"v1/main.rego": `package v1
+
+# comment
+allow := true
 `,
 		"unrelated.txt": `foobar`,
 	}
@@ -915,7 +1002,6 @@ func TestFixWithConflicts(t *testing.T) {
 
 	stdout := bytes.Buffer{}
 	stderr := bytes.Buffer{}
-	td := t.TempDir()
 
 	initialState := map[string]string{
 		".regal/config.yaml": "", // needed to find the root in the right place
@@ -944,9 +1030,7 @@ import rego.v1
 `,
 	}
 
-	for file, content := range initialState {
-		mustWriteToFile(t, filepath.Join(td, file), content)
-	}
+	td := testutil.TempDirectoryOf(t, initialState)
 
 	// --force is required to make the changes when there is no git repo
 	err := regal(&stdout, &stderr)("fix", "--force", td)
@@ -972,7 +1056,7 @@ Cannot move multiple files to: bar/bar.rego
 	}
 
 	for file, expectedContent := range initialState {
-		bs := testutil.Must(os.ReadFile(filepath.Join(td, file)))(t)
+		bs := testutil.MustReadFile(t, filepath.Join(td, file))
 
 		if act := string(bs); expectedContent != act {
 			t.Errorf("expected %s contents:\n%s\ngot\n%s", file, expectedContent, act)
@@ -985,7 +1069,6 @@ func TestFixWithConflictRenaming(t *testing.T) {
 
 	stdout := bytes.Buffer{}
 	stderr := bytes.Buffer{}
-	td := t.TempDir()
 
 	initialState := map[string]string{
 		".regal/config.yaml": "", // needed to find the root in the right place
@@ -1016,9 +1099,7 @@ import rego.v1
 `,
 	}
 
-	for file, content := range initialState {
-		mustWriteToFile(t, filepath.Join(td, file), content)
-	}
+	td := testutil.TempDirectoryOf(t, initialState)
 
 	// --force is required to make the changes when there is no git repo
 	// --conflict=rename will rename inbound files when there is a conflict
@@ -1090,6 +1171,78 @@ import rego.v1
 	}
 }
 
+func TestFixSingleFileNested(t *testing.T) {
+	t.Parallel()
+
+	stdout, stderr := bytes.Buffer{}, bytes.Buffer{}
+
+	initialState := map[string]string{
+		".regal/config.yaml": `
+project:
+  rego-version: 1
+`,
+		"foo/.regal.yaml": `
+project:
+  rego-version: 1
+rules:
+  style:
+    opa-fmt:
+      level: ignore
+`,
+		"foo/foo.rego": `package wow`,
+	}
+
+	td := testutil.TempDirectoryOf(t, initialState)
+
+	// --force is required to make the changes when there is no git repo
+	err := regal(&stdout, &stderr)(
+		"fix",
+		"--force",
+		filepath.Join(td, "foo", "foo.rego"),
+	)
+
+	// 0 exit status is expected as all violations should have been fixed
+	expectExitCode(t, err, 0, &stdout, &stderr)
+
+	exp := fmt.Sprintf(`1 fix applied:
+In project root: %[1]s
+foo.rego -> wow/foo.rego:
+- directory-package-mismatch
+`, filepath.Join(td, "foo"))
+
+	if act := stdout.String(); exp != act {
+		t.Fatalf("expected stdout:\n%s\ngot:\n%s", exp, act)
+	}
+
+	if exp, act := "", stderr.String(); exp != act {
+		t.Fatalf("expected stderr %q, got %q", exp, act)
+	}
+
+	expectedState := map[string]string{
+		".regal/config.yaml": `
+project:
+  rego-version: 1
+`,
+		"foo/.regal.yaml": `
+project:
+  rego-version: 1
+rules:
+  style:
+    opa-fmt:
+      level: ignore
+`,
+		"foo/wow/foo.rego": `package wow`,
+	}
+
+	for file, expectedContent := range expectedState {
+		bs := testutil.Must(os.ReadFile(filepath.Join(td, file)))(t)
+
+		if act := string(bs); expectedContent != act {
+			t.Errorf("expected %s contents:\n%s\ngot\n%s", file, expectedContent, act)
+		}
+	}
+}
+
 // verify fix for https://github.com/StyraInc/regal/issues/1082
 func TestLintAnnotationCustomAttributeMultipleItems(t *testing.T) {
 	t.Parallel()
@@ -1100,6 +1253,7 @@ func TestLintAnnotationCustomAttributeMultipleItems(t *testing.T) {
 
 	err := regal(&stdout, &stderr)(
 		"lint",
+		"--config-file", filepath.Join(cwd, "e2e_conf.yaml"),
 		"--disable=directory-package-mismatch",
 		filepath.Join(cwd, "testdata", "bugs", "issue_1082.rego"),
 	)
@@ -1182,15 +1336,3 @@ func expectExitCode(t *testing.T, err error, exp int, stdout *bytes.Buffer, stde
 			exp, act, stdout.String(), stderr.String())
 	}
 }
-
-func mustWriteToFile(t *testing.T, path string, content string) {
-	t.Helper()
-
-	if err := os.MkdirAll(filepath.Dir(path), 0o755); err != nil {
-		t.Fatalf("failed to create directory %s: %v", filepath.Dir(path), err)
-	}
-
-	if err := os.WriteFile(path, []byte(content), 0o644); err != nil {
-		t.Fatalf("failed to write to %s: %v", path, err)
-	}
-}
