@@ -8,11 +8,17 @@ import (
 	"os"
 	"path/filepath"
 	"strings"
+	"sync"
 
-	"github.com/anderseknert/roast/pkg/encoding"
+	"gopkg.in/yaml.v3"
 
-	"github.com/open-policy-agent/opa/bundle"
-	"github.com/open-policy-agent/opa/loader/filter"
+	"github.com/open-policy-agent/opa/v1/ast"
+	"github.com/open-policy-agent/opa/v1/bundle"
+	"github.com/open-policy-agent/opa/v1/loader/filter"
+	"github.com/open-policy-agent/opa/v1/rego"
+	"github.com/open-policy-agent/opa/v1/types"
+
+	"github.com/styrainc/roast/pkg/encoding"
 )
 
 const PathSeparator = string(os.PathSeparator)
@@ -26,6 +32,7 @@ func LoadRegalBundleFS(fs files.FS) (bundle.Bundle, error) {
 
 	//nolint:wrapcheck
 	return bundle.NewCustomReader(embedLoader.WithFilter(ExcludeTestFilter())).
+		WithCapabilities(Capabilities()).
 		WithSkipBundleVerification(true).
 		WithProcessAnnotations(true).
 		WithBundleName("regal").
@@ -36,6 +43,7 @@ func LoadRegalBundleFS(fs files.FS) (bundle.Bundle, error) {
 func LoadRegalBundlePath(path string) (bundle.Bundle, error) {
 	//nolint:wrapcheck
 	return bundle.NewCustomReader(bundle.NewDirectoryLoader(path).WithFilter(ExcludeTestFilter())).
+		WithCapabilities(Capabilities()).
 		WithSkipBundleVerification(true).
 		WithProcessAnnotations(true).
 		WithBundleName("regal").
@@ -56,30 +64,11 @@ func MustLoadRegalBundleFS(fs files.FS) bundle.Bundle {
 func ToMap(a any) map[string]any {
 	r := make(map[string]any)
 
-	MustJSONRoundTrip(a, &r)
+	encoding.MustJSONRoundTrip(a, &r)
 
 	return r
 }
 
-// JSONRoundTrip convert any value to JSON and back again.
-func JSONRoundTrip(from any, to any) error {
-	json := encoding.JSON()
-
-	bs, err := json.Marshal(from)
-	if err != nil {
-		return fmt.Errorf("failed JSON marshalling %w", err)
-	}
-
-	return json.Unmarshal(bs, to) //nolint:wrapcheck
-}
-
-// MustJSONRoundTrip convert any value to JSON and back again, exit on failure.
-func MustJSONRoundTrip(from any, to any) {
-	if err := JSONRoundTrip(from, to); err != nil {
-		log.Fatal(err)
-	}
-}
-
 // CloseFileIgnore closes file ignoring errors, mainly for deferred cleanup.
 func CloseFileIgnore(file *os.File) {
 	_ = file.Close()
@@ -94,22 +83,61 @@ func ExcludeTestFilter() filter.LoaderFilter {
 	}
 }
 
-// FindInput finds input.json file in workspace closest to the file, and returns
-// both the location and the reader.
-func FindInput(file string, workspacePath string) (string, io.Reader) {
+// FindInput finds input.json or input.yaml file in workspace closest to the file, and returns
+// both the location and the contents of the file (as map), or an empty string and nil if not found.
+// Note that:
+// - This function doesn't do error handling. If the file can't be read, nothing is returned.
+// - While the input data theoritcally could be anything JSON/YAML value, we only support an object.
+func FindInput(file string, workspacePath string) (string, map[string]any) {
 	relative := strings.TrimPrefix(file, workspacePath)
-	components := strings.Split(filepath.Dir(relative), string(filepath.Separator))
+	components := strings.Split(filepath.Dir(relative), PathSeparator)
+
+	var (
+		inputPath string
+		content   []byte
+	)
 
 	for i := range components {
-		inputPath := filepath.Join(workspacePath, filepath.Join(components[:len(components)-i]...), "input.json")
+		current := components[:len(components)-i]
 
-		f, err := os.Open(inputPath)
+		inputPathJSON := filepath.Join(workspacePath, filepath.Join(current...), "input.json")
+
+		f, err := os.Open(inputPathJSON)
 		if err == nil {
-			return inputPath, f
+			inputPath = inputPathJSON
+			content, _ = io.ReadAll(f)
+
+			break
+		}
+
+		inputPathYAML := filepath.Join(workspacePath, filepath.Join(current...), "input.yaml")
+
+		f, err = os.Open(inputPathYAML)
+		if err == nil {
+			inputPath = inputPathYAML
+			content, _ = io.ReadAll(f)
+
+			break
 		}
 	}
 
-	return "", nil
+	if inputPath == "" || content == nil {
+		return "", nil
+	}
+
+	var input map[string]any
+
+	if strings.HasSuffix(inputPath, ".json") {
+		if err := encoding.JSON().Unmarshal(content, &input); err != nil {
+			return "", nil
+		}
+	} else if strings.HasSuffix(inputPath, ".yaml") {
+		if err := yaml.Unmarshal(content, &input); err != nil {
+			return "", nil
+		}
+	}
+
+	return inputPath, input
 }
 
 func IsSkipWalkDirectory(info files.DirEntry) bool {
@@ -121,7 +149,7 @@ func IsSkipWalkDirectory(info files.DirEntry) bool {
 // are passed to the callback, and where directories that should commonly  be ignored
 // (.git, node_modules, etc.) are skipped.
 func WalkFiles(root string, f func(path string) error) error {
-	return filepath.WalkDir(root, func(path string, info os.DirEntry, _ error) error { // nolint:wrapcheck
+	return filepath.WalkDir(root, func(path string, info os.DirEntry, _ error) error { //nolint:wrapcheck
 		if IsSkipWalkDirectory(info) {
 			return filepath.SkipDir
 		}
@@ -153,3 +181,53 @@ func FindManifestLocations(root string) ([]string, error) {
 
 	return foundBundleRoots, nil
 }
+
+// NOTE: These are mirrored here merely to provide correct capabilities for the
+// parser. Can't import other packages from here as almost all of them depend on
+// this package. We should probably move these definitions to somewhere where all
+// packages can import them from.
+
+//nolint:gochecknoglobals
+var regalParseModuleMeta = &rego.Function{
+	Name: "regal.parse_module",
+	Decl: types.NewFunction(
+		types.Args(
+			types.Named("filename", types.S).Description("file name to attach to AST nodes' locations"),
+			types.Named("rego", types.S).Description("Rego module"),
+		),
+		types.Named("output", types.NewObject(nil, types.NewDynamicProperty(types.S, types.A))),
+	),
+}
+
+//nolint:gochecknoglobals
+var regalLastMeta = &rego.Function{
+	Name: "regal.last",
+	Decl: types.NewFunction(
+		types.Args(
+			types.Named("array", types.NewArray(nil, types.A)).
+				Description("performance optimized last index retrieval"),
+		),
+		types.Named("element", types.A),
+	),
+}
+
+//nolint:gochecknoglobals
+var OPACapabilities = ast.CapabilitiesForThisVersion()
+
+//nolint:gochecknoglobals
+var Capabilities = sync.OnceValue(capabilities)
+
+func capabilities() *ast.Capabilities {
+	cpy := *OPACapabilities
+	cpy.Builtins = append(cpy.Builtins,
+		&ast.Builtin{
+			Name: regalParseModuleMeta.Name,
+			Decl: regalParseModuleMeta.Decl,
+		},
+		&ast.Builtin{
+			Name: regalLastMeta.Name,
+			Decl: regalLastMeta.Decl,
+		})
+
+	return &cpy
+}
