@@ -6,20 +6,19 @@ import (
 	"path/filepath"
 	"strconv"
 	"strings"
-	"sync"
 
-	"github.com/open-policy-agent/opa/ast"
-	"github.com/open-policy-agent/opa/types"
+	"github.com/open-policy-agent/opa/v1/ast"
+	"github.com/open-policy-agent/opa/v1/types"
 
 	"github.com/styrainc/regal/internal/lsp/cache"
 	"github.com/styrainc/regal/internal/lsp/examples"
 	"github.com/styrainc/regal/internal/lsp/rego"
 	types2 "github.com/styrainc/regal/internal/lsp/types"
+
+	"github.com/styrainc/roast/pkg/util/concurrent"
 )
 
-var builtinCache = make(map[*ast.Builtin]string) //nolint:gochecknoglobals
-
-var builtinCacheLock = &sync.Mutex{} //nolint:gochecknoglobals
+var builtinCache = concurrent.MapOf(make(map[*ast.Builtin]string)) //nolint:gochecknoglobals
 
 func writeFunctionSnippet(sb *strings.Builder, builtin *ast.Builtin) {
 	sb.WriteString("```rego\n")
@@ -52,13 +51,9 @@ func writeFunctionSnippet(sb *strings.Builder, builtin *ast.Builtin) {
 }
 
 func CreateHoverContent(builtin *ast.Builtin) string {
-	builtinCacheLock.Lock()
-	if content, ok := builtinCache[builtin]; ok {
-		builtinCacheLock.Unlock()
-
+	if content, ok := builtinCache.Get(builtin); ok {
 		return content
 	}
-	builtinCacheLock.Unlock()
 
 	link := fmt.Sprintf(
 		"https://www.openpolicyagent.org/docs/latest/policy-reference/#builtin-%s-%s",
@@ -139,9 +134,7 @@ func CreateHoverContent(builtin *ast.Builtin) string {
 
 	result := sb.String()
 
-	builtinCacheLock.Lock()
-	builtinCache[builtin] = result
-	builtinCacheLock.Unlock()
+	builtinCache.Set(builtin, result)
 
 	return result
 }
@@ -154,7 +147,7 @@ func UpdateBuiltinPositions(cache *cache.Cache, uri string, builtins map[string]
 
 	builtinsOnLine := map[uint][]types2.BuiltinPosition{}
 
-	// nolint:gosec
+	//nolint:gosec
 	for _, call := range rego.AllBuiltinCalls(module, builtins) {
 		line := uint(call.Location.Row)
 
