@@ -9,14 +9,17 @@ import (
 	"slices"
 	"strings"
 
-	"github.com/anderseknert/roast/pkg/encoding"
 	"gopkg.in/yaml.v3"
 
-	"github.com/open-policy-agent/opa/ast"
+	"github.com/open-policy-agent/opa/v1/ast"
+	"github.com/open-policy-agent/opa/v1/bundle"
 
 	"github.com/styrainc/regal/internal/capabilities"
 	rio "github.com/styrainc/regal/internal/io"
 	"github.com/styrainc/regal/internal/util"
+
+	"github.com/styrainc/roast/pkg/encoding"
+	rutil "github.com/styrainc/roast/pkg/util"
 )
 
 const (
@@ -38,8 +41,18 @@ type Config struct {
 	Ignore          Ignore              `json:"ignore,omitempty"           yaml:"ignore,omitempty"`
 }
 
+type Root struct {
+	Path string
+	// Note that contrary to ast.RegoVersion, we'll only accept 0 or 1 here currently.
+	// This aligns with the Rego versioning scheme used in .manifest files, which is
+	// the alternative way to provide this for a specific directory.
+	RegoVersion *int `json:"rego-version,omitempty" yaml:"rego-version,omitempty"`
+}
+
 type Project struct {
-	Roots []string `json:"roots" yaml:"roots"`
+	Roots *[]Root `json:"roots,omitempty" yaml:"roots,omitempty"`
+	// Set the Rego version for the whole project or workspace. Individual roots may override this.
+	RegoVersion *int `json:"rego-version,omitempty" yaml:"rego-version,omitempty"`
 }
 
 type Category map[string]Rule
@@ -107,20 +120,66 @@ type Builtin struct {
 }
 
 const (
-	regalDirName   = ".regal"
-	configFileName = "config.yaml"
+	regalDirName             = ".regal"
+	configFileName           = "config.yaml"
+	standaloneConfigFileName = ".regal.yaml"
 )
 
-// FindRegalDirectory searches for a .regal directory first in the directory of path, and if not found,
-// in the parent directory, and if not found, in the parent's parent directory, and so on.
-func FindRegalDirectory(path string) (*os.File, error) {
+// FindConfig attempts to find either the .regal directory or .regal.yaml
+// config file, and returns the appropriate file or an error.
+func FindConfig(path string) (*os.File, error) {
+	regalDir, regalDirError := FindRegalDirectory(path)
+	regalConfigFile, regalConfigFileError := FindRegalConfigFile(path)
+
+	var regalDirParent, regalConfigFileParent string
+	if regalDirError == nil && regalConfigFileError == nil {
+		regalDirParent = filepath.Dir(regalDir.Name())
+		regalConfigFileParent = filepath.Dir(regalConfigFile.Name())
+
+		if regalDirParent == regalConfigFileParent {
+			return nil, errors.New("conflicting config files: both .regal directory and .regal.yaml found")
+		}
+	}
+
+	if regalDirError != nil && regalConfigFileError != nil {
+		return nil, errors.New("could not find Regal config")
+	}
+
+	// if the config file parent is not "", then it was found, and if it's
+	// longer then it's more specific to the search path in question, and so we
+	// return that here in such cases.
+	if len(regalConfigFileParent) > len(regalDirParent) {
+		return regalConfigFile, nil
+	}
+
+	// if there is a .regal directory, when a config file is expected to be
+	// found inside.
+	if regalDirError == nil {
+		expectedConfigFilePath := filepath.Join(regalDir.Name(), rio.PathSeparator, configFileName)
+
+		_, err := os.Stat(expectedConfigFilePath)
+		if err != nil && os.IsNotExist(err) {
+			return nil, errors.New("config file was not found in .regal directory")
+		} else if err != nil {
+			return nil, fmt.Errorf("failed to stat .regal config file: %w", err)
+		}
+
+		return os.Open(expectedConfigFilePath) //nolint:wrapcheck
+	}
+
+	// regalConfigFileError is nil at this point, so we can return the file
+	return regalConfigFile, nil
+}
+
+// findUpwards searches for a file or directory matching the given name,
+// starting from the provided path and moving upwards.
+func findUpwards(path, name string, expectDir bool) (*os.File, error) {
 	finfo, err := os.Stat(path)
 	if err != nil {
 		return nil, fmt.Errorf("failed to stat path %v: %w", path, err)
 	}
 
 	dir := path
-
 	if !finfo.IsDir() {
 		dir = filepath.Dir(path)
 	}
@@ -131,33 +190,31 @@ func FindRegalDirectory(path string) (*os.File, error) {
 	for {
 		var searchPath string
 		if volume == "" {
-			searchPath = filepath.Join(rio.PathSeparator, dir, regalDirName)
+			searchPath = filepath.Join(rio.PathSeparator, dir, name)
 		} else {
-			searchPath = filepath.Join(dir, regalDirName)
+			searchPath = filepath.Join(dir, name)
 		}
 
-		regalDir, err := os.Open(searchPath)
+		file, err := os.Open(searchPath)
 		if err == nil {
-			rdInfo, err := regalDir.Stat()
-			if err == nil && rdInfo.IsDir() {
-				return regalDir, nil
+			fileInfo, err := file.Stat()
+			if err == nil && fileInfo.IsDir() == expectDir {
+				return file, nil
 			}
 		}
 
-		if searchPath == volume+rio.PathSeparator+regalDirName {
+		if searchPath == volume+rio.PathSeparator+name {
 			// Stop traversing at the root path
 			return nil, fmt.Errorf("can't traverse past root directory %w", err)
 		}
 
 		// Move up one level in the directory tree
 		parts := strings.Split(dir, rio.PathSeparator)
-
 		if len(parts) < 2 {
 			return nil, errors.New("stopping as dir is root directory")
 		}
 
 		parts = parts[:len(parts)-1]
-
 		if parts[0] == volume {
 			parts[0] = volume + rio.PathSeparator
 		}
@@ -166,6 +223,16 @@ func FindRegalDirectory(path string) (*os.File, error) {
 	}
 }
 
+// FindRegalDirectory searches for a .regal directory upwards from the provided path.
+func FindRegalDirectory(path string) (*os.File, error) {
+	return findUpwards(path, regalDirName, true)
+}
+
+// FindRegalConfigFile searches for a .regal.yaml config file upwards from the provided path.
+func FindRegalConfigFile(path string) (*os.File, error) {
+	return findUpwards(path, standaloneConfigFileName, false)
+}
+
 // FindBundleRootDirectories finds all bundle root directories from the provided path,
 // which **must** be an absolute path. Bundle root directories may be found either by:
 //
@@ -181,7 +248,7 @@ func FindBundleRootDirectories(path string) ([]string, error) {
 	// This will traverse the tree **upwards** searching for a .regal directory
 	regalDir, err := FindRegalDirectory(path)
 	if err == nil {
-		roots, err := rootsFromRegalDirectory(regalDir)
+		roots, err := rootsFromRegalConfigDirOrFile(regalDir)
 		if err != nil {
 			return nil, fmt.Errorf("failed to get roots from .regal directory: %w", err)
 		}
@@ -189,6 +256,17 @@ func FindBundleRootDirectories(path string) ([]string, error) {
 		foundBundleRoots = append(foundBundleRoots, roots...)
 	}
 
+	// This will traverse the tree **upwards** searching for a .regal.yaml file
+	regalConfigFile, err := FindRegalConfigFile(path)
+	if err == nil {
+		roots, err := rootsFromRegalConfigDirOrFile(regalConfigFile)
+		if err != nil {
+			return nil, fmt.Errorf("failed to get roots from .regal.yaml: %w", err)
+		}
+
+		foundBundleRoots = append(foundBundleRoots, roots...)
+	}
+
 	// This will traverse the tree **downwards** searching for .regal directories
 	// Not using rio.WalkFiles here as we're specifically looking for directories
 	if err := filepath.WalkDir(path, func(path string, info os.DirEntry, err error) error {
@@ -207,7 +285,7 @@ func FindBundleRootDirectories(path string) ([]string, error) {
 
 			defer rd.Close()
 
-			roots, err := rootsFromRegalDirectory(rd)
+			roots, err := rootsFromRegalConfigDirOrFile(rd)
 			if err != nil {
 				return fmt.Errorf("failed to get roots from .regal directory: %w", err)
 			}
@@ -230,39 +308,60 @@ func FindBundleRootDirectories(path string) ([]string, error) {
 	return slices.Compact(foundBundleRoots), nil
 }
 
-func rootsFromRegalDirectory(regalDir *os.File) ([]string, error) {
-	foundBundleRoots := make([]string, 0)
+func rootsFromRegalConfigDirOrFile(file *os.File) ([]string, error) {
+	defer file.Close()
 
-	defer regalDir.Close()
+	fileInfo, err := file.Stat()
+	if err != nil {
+		return nil, fmt.Errorf("failed to stat file: %w", err)
+	}
 
-	parent, _ := filepath.Split(regalDir.Name())
+	if (fileInfo.IsDir() && filepath.Base(file.Name()) != ".regal") ||
+		(!fileInfo.IsDir() && filepath.Base(file.Name()) != ".regal.yaml") {
+		return nil, fmt.Errorf(
+			"expected a directory named '.regal' or a file named '.regal.yaml', got '%s'",
+			filepath.Base(file.Name()),
+		)
+	}
 
-	parent = filepath.Clean(parent)
+	parent := filepath.Dir(file.Name())
 
-	// add the parent directory of .regal
-	foundBundleRoots = append(foundBundleRoots, parent)
+	foundBundleRoots := []string{parent}
 
-	file, err := os.ReadFile(filepath.Join(regalDir.Name(), "config.yaml"))
+	var configFilePath string
+
+	if fileInfo.IsDir() {
+		configFilePath = filepath.Join(file.Name(), "config.yaml")
+	} else {
+		configFilePath = file.Name()
+	}
+
+	fileContent, err := os.ReadFile(configFilePath)
 	if err == nil {
 		var conf Config
-
-		if err = yaml.Unmarshal(file, &conf); err != nil {
+		if err = yaml.Unmarshal(fileContent, &conf); err != nil {
 			return nil, fmt.Errorf("failed to unmarshal config file: %w", err)
 		}
 
-		if conf.Project != nil {
-			foundBundleRoots = append(foundBundleRoots, util.Map(util.FilepathJoiner(parent), conf.Project.Roots)...)
+		if conf.Project != nil && conf.Project.Roots != nil {
+			for _, root := range *conf.Project.Roots {
+				foundBundleRoots = append(foundBundleRoots, filepath.Join(parent, root.Path))
+			}
 		}
 	}
 
-	customRulesDir := filepath.Join(regalDir.Name(), "rules")
+	// Include the "rules" directory when loading from a .regal dir
+	if fileInfo.IsDir() {
+		customDir := filepath.Join(file.Name(), "rules")
 
-	info, err := os.Stat(customRulesDir)
-	if err == nil && info.IsDir() {
-		foundBundleRoots = append(foundBundleRoots, customRulesDir)
+		info, err := os.Stat(customDir)
+		if err == nil && info.IsDir() {
+			foundBundleRoots = append(foundBundleRoots, customDir)
+		}
 	}
 
-	manifestRoots, err := rio.FindManifestLocations(filepath.Dir(regalDir.Name()))
+	// Include a search for manifest files
+	manifestRoots, err := rio.FindManifestLocations(parent)
 	if err != nil {
 		return nil, fmt.Errorf("failed while looking for manifest locations: %w", err)
 	}
@@ -272,29 +371,90 @@ func rootsFromRegalDirectory(regalDir *os.File) ([]string, error) {
 	return foundBundleRoots, nil
 }
 
-func FindConfig(path string) (*os.File, error) {
-	regalDir, err := FindRegalDirectory(path)
-	if err != nil {
-		return nil, fmt.Errorf("could not find .regal directory: %w", err)
-	}
-
-	return os.Open(filepath.Join(regalDir.Name(), rio.PathSeparator, configFileName)) //nolint:wrapcheck
-}
-
 func FromMap(confMap map[string]any) (Config, error) {
 	var conf Config
 
-	if err := rio.JSONRoundTrip(confMap, &conf); err != nil {
+	if err := encoding.JSONRoundTrip(confMap, &conf); err != nil {
 		return conf, fmt.Errorf("failed to convert config map to config struct: %w", err)
 	}
 
 	return conf, nil
 }
 
+// AllRegoVersions returns a map of all Rego versions found in the provided config and .manifest files,
+// keyed by the path of the directory for which the version applies. Config file has higher precedence
+// than .manifest files.
+func AllRegoVersions(root string, conf *Config) (map[string]ast.RegoVersion, error) {
+	versionsMap := make(map[string]ast.RegoVersion)
+
+	if conf == nil {
+		return versionsMap, nil
+	}
+
+	manifestLocations, err := rio.FindManifestLocations(root)
+	if err != nil {
+		return nil, fmt.Errorf("failed to find manifest locations: %w", err)
+	}
+
+	for _, dir := range manifestLocations {
+		manifestPath := filepath.Join(root, dir, ".manifest")
+
+		f, err := os.ReadFile(manifestPath)
+		if err != nil {
+			return nil, fmt.Errorf("failed to read manifest file: %w", err)
+		}
+
+		var manifest bundle.Manifest
+
+		err = encoding.JSON().Unmarshal(f, &manifest)
+		if err != nil {
+			return nil, fmt.Errorf("failed to unmarshal manifest file: %w", err)
+		}
+
+		if manifest.RegoVersion != nil {
+			versionsMap[dir] = regoVersionFromConfigValue(manifest.RegoVersion)
+		}
+	}
+
+	if conf.Project != nil {
+		if conf.Project.RegoVersion != nil {
+			// empty string is used to denote the project root
+			// and all other paths are relative to that
+			versionsMap[""] = regoVersionFromConfigValue(conf.Project.RegoVersion)
+		}
+
+		if conf.Project.Roots != nil {
+			for _, root := range *conf.Project.Roots {
+				if root.RegoVersion != nil {
+					versionsMap[root.Path] = regoVersionFromConfigValue(root.RegoVersion)
+				}
+			}
+		}
+	}
+
+	return versionsMap, nil
+}
+
+// Translate integer value 0 or 1 as found in config or .manifest to ast.RegoVersion.
+func regoVersionFromConfigValue(version *int) ast.RegoVersion {
+	if version == nil {
+		return ast.RegoUndefined
+	}
+
+	switch *version {
+	case 0:
+		return ast.RegoV0
+	case 1:
+		return ast.RegoV1
+	default:
+		return ast.RegoUndefined
+	}
+}
+
 func (config Config) MarshalYAML() (any, error) {
 	var unstructuredConfig map[string]any
 
-	if err := rio.JSONRoundTrip(config, &unstructuredConfig); err != nil {
+	if err := encoding.JSONRoundTrip(config, &unstructuredConfig); err != nil {
 		return nil, fmt.Errorf("failed to created unstructured config: %w", err)
 	}
 
@@ -361,6 +521,28 @@ type marshallingIntermediary struct {
 	} `yaml:"features"`
 }
 
+func (p *Project) UnmarshalYAML(value *yaml.Node) error {
+	var result map[string]any
+
+	if err := value.Decode(&result); err != nil {
+		return fmt.Errorf("unmarshalling project failed %w", err)
+	}
+
+	if result["roots"] != nil {
+		// A project root can either be provided as a string or an object. If it's a string, we'll
+		// convert it to an object with the path as the only value.
+		if roots, ok := result["roots"].([]any); ok {
+			for key, val := range roots {
+				if root, ok := val.(string); ok {
+					roots[key] = Root{Path: root}
+				}
+			}
+		}
+	}
+
+	return encoding.JSONRoundTrip(result, p) //nolint:wrapcheck
+}
+
 func (config *Config) UnmarshalYAML(value *yaml.Node) error {
 	var result marshallingIntermediary
 
@@ -451,7 +633,7 @@ func (config *Config) UnmarshalYAML(value *yaml.Node) error {
 		return fmt.Errorf("failed to load capabilities: %w", err)
 	}
 
-	config.Capabilities = fromOPACapabilities(*opaCaps)
+	config.Capabilities = fromOPACapabilities(opaCaps)
 
 	// This is used in the LSP to load the OPA capabilities, since the
 	// capabilities version in the user-facing config does not contain all
@@ -556,7 +738,7 @@ func extractDefaults(c *Config, result *marshallingIntermediary) error {
 
 // CapabilitiesForThisVersion returns the capabilities for the current OPA version Regal depends on.
 func CapabilitiesForThisVersion() *Capabilities {
-	return fromOPACapabilities(*ast.CapabilitiesForThisVersion())
+	return fromOPACapabilities(rio.OPACapabilities)
 }
 
 func fromOPABuiltin(builtin ast.Builtin) *Builtin {
@@ -577,10 +759,10 @@ func fromOPABuiltin(builtin ast.Builtin) *Builtin {
 	return rb
 }
 
-func fromOPACapabilities(capabilities ast.Capabilities) *Capabilities {
+func fromOPACapabilities(capabilities *ast.Capabilities) *Capabilities {
 	var result Capabilities
 
-	result.Builtins = make(map[string]*Builtin)
+	result.Builtins = make(map[string]*Builtin, len(capabilities.Builtins))
 
 	for _, builtin := range capabilities.Builtins {
 		result.Builtins[builtin.Name] = fromOPABuiltin(*builtin)
@@ -595,7 +777,7 @@ func fromOPACapabilities(capabilities ast.Capabilities) *Capabilities {
 func ToMap(config Config) map[string]any {
 	confMap := make(map[string]any)
 
-	rio.MustJSONRoundTrip(config, &confMap)
+	encoding.MustJSONRoundTrip(config, &confMap)
 
 	// Not sure why `omitempty` doesn't do the trick here, but having `ignore: {}` in the config for each
 	// rule is annoying noice when printed from Rego.
@@ -612,7 +794,7 @@ func ToMap(config Config) map[string]any {
 	return confMap
 }
 
-func (rule Rule) MarshalJSON() ([]byte, error) {
+func (rule *Rule) MarshalJSON() ([]byte, error) {
 	result, err := rule.MarshalYAML()
 	if err != nil {
 		return nil, fmt.Errorf("marshalling rule failed %w", err)
@@ -635,7 +817,7 @@ func (rule *Rule) UnmarshalJSON(data []byte) error {
 	return rule.mapToConfig(result)
 }
 
-func (rule Rule) MarshalYAML() (interface{}, error) {
+func (rule *Rule) MarshalYAML() (interface{}, error) {
 	result := make(map[string]any)
 	result[keyLevel] = rule.Level
 
@@ -658,7 +840,7 @@ func (rule *Rule) UnmarshalYAML(value *yaml.Node) error {
 		return fmt.Errorf("unmarshalling rule failed %w", err)
 	}
 
-	return rule.mapToConfig(result) //nolint:errcheck
+	return rule.mapToConfig(result)
 }
 
 // Note that this function will mutate the result map. This isn't a problem right now
@@ -678,7 +860,7 @@ func (rule *Rule) mapToConfig(result any) error {
 	if ignore, ok := ruleMap[keyIgnore]; ok {
 		var dst Ignore
 
-		if err := rio.JSONRoundTrip(ignore, &dst); err != nil {
+		if err := encoding.JSONRoundTrip(ignore, &dst); err != nil {
 			return fmt.Errorf("unmarshalling rule ignore failed: %w", err)
 		}
 
@@ -696,8 +878,6 @@ func (rule *Rule) mapToConfig(result any) error {
 func GetPotentialRoots(paths ...string) ([]string, error) {
 	var err error
 
-	dirMap := make(map[string]struct{})
-
 	absDirPaths := make([]string, len(paths))
 
 	for i, path := range paths {
@@ -717,23 +897,22 @@ func GetPotentialRoots(paths ...string) ([]string, error) {
 		}
 	}
 
+	dirMap := rutil.NewSet[string]()
+
 	for _, dir := range absDirPaths {
 		brds, err := FindBundleRootDirectories(dir)
 		if err != nil {
 			return nil, fmt.Errorf("failed to find bundle root directories in %s: %w", dir, err)
 		}
 
-		for _, brd := range brds {
-			dirMap[brd] = struct{}{}
-		}
+		dirMap.Add(brds...)
 	}
 
-	foundRoots := util.Keys(dirMap)
-	if len(foundRoots) == 0 {
+	if dirMap.Size() == 0 {
 		return absDirPaths, nil
 	}
 
-	return foundRoots, nil
+	return dirMap.Items(), nil
 }
 
 func isDir(path string) bool {
