@@ -1,19 +1,20 @@
 package fixer
 
 import (
-	"bytes"
 	"context"
 	"errors"
 	"fmt"
 	"path/filepath"
 	"slices"
 
-	"github.com/open-policy-agent/opa/ast"
+	"github.com/open-policy-agent/opa/v1/ast"
 
 	"github.com/styrainc/regal/internal/util"
+	"github.com/styrainc/regal/pkg/config"
 	"github.com/styrainc/regal/pkg/fixer/fileprovider"
 	"github.com/styrainc/regal/pkg/fixer/fixes"
 	"github.com/styrainc/regal/pkg/linter"
+	"github.com/styrainc/regal/pkg/report"
 )
 
 type OnConflictOperation string
@@ -26,19 +27,18 @@ const (
 // NewFixer instantiates a Fixer.
 func NewFixer() *Fixer {
 	return &Fixer{
-		registeredFixes:          make(map[string]any),
-		registeredMandatoryFixes: make(map[string]any),
-		registeredRoots:          make([]string, 0),
-		onConflictOperation:      OnConflictError,
+		registeredFixes:     make(map[string]any),
+		registeredRoots:     make([]string, 0),
+		onConflictOperation: OnConflictError,
 	}
 }
 
 // Fixer must be instantiated via NewFixer.
 type Fixer struct {
-	registeredFixes          map[string]any
-	registeredMandatoryFixes map[string]any
-	onConflictOperation      OnConflictOperation
-	registeredRoots          []string
+	registeredFixes     map[string]any
+	onConflictOperation OnConflictOperation
+	registeredRoots     []string
+	versionsMap         map[string]ast.RegoVersion
 }
 
 // SetOnConflictOperation sets the fixer's behavior when a conflict occurs.
@@ -46,24 +46,17 @@ func (f *Fixer) SetOnConflictOperation(operation OnConflictOperation) {
 	f.onConflictOperation = operation
 }
 
+// SetRegoVersionsMap sets the mapping of path prefixes to versions for the
+// fixer to use when creating input for fixer runs.
+func (f *Fixer) SetRegoVersionsMap(versionsMap map[string]ast.RegoVersion) {
+	f.versionsMap = versionsMap
+}
+
 // RegisterFixes sets the fixes that will be fixed if there are related linter
 // violations that can be fixed by fixes.
 func (f *Fixer) RegisterFixes(fixes ...fixes.Fix) {
 	for _, fix := range fixes {
-		if _, mandatory := f.registeredMandatoryFixes[fix.Name()]; !mandatory {
-			f.registeredFixes[fix.Name()] = fix
-		}
-	}
-}
-
-// RegisterMandatoryFixes sets fixes which will be run before other registered
-// fixes, against all files which are not ignored, regardless of linter
-// violations.
-func (f *Fixer) RegisterMandatoryFixes(fixes ...fixes.Fix) {
-	for _, fix := range fixes {
-		f.registeredMandatoryFixes[fix.Name()] = fix
-
-		delete(f.registeredFixes, fix.Name())
+		f.registeredFixes[fix.Name()] = fix
 	}
 }
 
@@ -89,33 +82,9 @@ func (f *Fixer) GetFixForName(name string) (fixes.Fix, bool) {
 	return fixInstance, true
 }
 
-func (f *Fixer) GetMandatoryFixForName(name string) (fixes.Fix, bool) {
-	fix, ok := f.registeredMandatoryFixes[name]
-	if !ok {
-		return nil, false
-	}
-
-	fixInstance, ok := fix.(fixes.Fix)
-	if !ok {
-		return nil, false
-	}
-
-	return fixInstance, true
-}
-
 func (f *Fixer) Fix(ctx context.Context, l *linter.Linter, fp fileprovider.FileProvider) (*Report, error) {
 	fixReport := NewReport()
 
-	// Early return if there are no registered mandatory fixes
-	if len(f.registeredMandatoryFixes) == 0 && len(f.registeredFixes) == 0 {
-		return fixReport, nil
-	}
-
-	// Apply mandatory fixes
-	if err := f.applyMandatoryFixes(fp, fixReport); err != nil {
-		return nil, err
-	}
-
 	// If there are no registered fixes that require a linter, return the report
 	if len(f.registeredFixes) == 0 {
 		return fixReport, nil
@@ -129,64 +98,74 @@ func (f *Fixer) Fix(ctx context.Context, l *linter.Linter, fp fileprovider.FileP
 	return fixReport, nil
 }
 
-// applyMandatoryFixes handles the application of mandatory fixes to all files.
-func (f *Fixer) applyMandatoryFixes(fp fileprovider.FileProvider, fixReport *Report) error {
-	if len(f.registeredMandatoryFixes) == 0 {
-		return nil
+func (f *Fixer) FixViolations(
+	violations []report.Violation,
+	fp fileprovider.FileProvider,
+	config *config.Config,
+) (*Report, error) {
+	fixReport := NewReport()
+
+	startingFiles, err := fp.List()
+	if err != nil {
+		return nil, fmt.Errorf("failed to list files: %w", err)
 	}
 
-	for {
-		fixMadeInIteration := false
+	for _, violation := range violations {
+		file := violation.Location.File
 
-		files, err := fp.List()
-		if err != nil {
-			return fmt.Errorf("failed to list files: %w", err)
+		fixInstance, ok := f.GetFixForName(violation.Title)
+		if !ok {
+			return nil, fmt.Errorf("no fix for violation %s", violation.Title)
 		}
 
-		for _, file := range files {
-			for fix := range f.registeredMandatoryFixes {
-				fixInstance, ok := f.GetMandatoryFixForName(fix)
-				if !ok {
-					return fmt.Errorf("no mandatory fix matched %s", fix)
-				}
+		fc, err := fp.Get(file)
+		if err != nil {
+			return nil, fmt.Errorf("failed to get file %s: %w", file, err)
+		}
 
-				fc, err := fp.Get(file)
-				if err != nil {
-					return fmt.Errorf("failed to get file %s: %w", file, err)
-				}
+		fixCandidate := fixes.FixCandidate{
+			Filename: file,
+			Contents: fc,
+		}
 
-				fixCandidate := fixes.FixCandidate{
-					Filename: file,
-					Contents: fc,
-				}
+		abs, err := filepath.Abs(file)
+		if err != nil {
+			return nil, fmt.Errorf("failed to get absolute path for %s: %w", file, err)
+		}
 
-				fixResults, err := fixInstance.Fix(&fixCandidate, &fixes.RuntimeOptions{
-					BaseDir: util.FindClosestMatchingRoot(file, f.registeredRoots),
-				})
-				if err != nil {
-					return fmt.Errorf("failed to fix %s: %w", file, err)
-				}
+		fixResults, err := fixInstance.Fix(&fixCandidate, &fixes.RuntimeOptions{
+			BaseDir: util.FindClosestMatchingRoot(abs, f.registeredRoots),
+			Config:  config,
+			Locations: []report.Location{
+				violation.Location,
+			},
+		})
+		if err != nil {
+			return nil, fmt.Errorf("failed to fix %s: %w", file, err)
+		}
 
-				for _, fixResult := range fixResults {
-					if !bytes.Equal(fc, fixResult.Contents) {
-						if err := fp.Put(file, fixResult.Contents); err != nil {
-							return fmt.Errorf("failed to write fixed rego for file %s: %w", file, err)
-						}
+		if len(fixResults) == 0 {
+			continue
+		}
 
-						fixReport.AddFileFix(file, fixResult)
+		fixResult := fixResults[0]
 
-						fixMadeInIteration = true
-					}
-				}
+		if fixResult.Rename != nil {
+			err = f.handleRename(fp, fixReport, startingFiles, fixResult)
+			if err != nil {
+				return nil, fmt.Errorf("failed to handle rename: %w", err)
 			}
 		}
 
-		if !fixMadeInIteration {
-			break
+		// Write the fixed content to the file
+		if err := fp.Put(file, fixResult.Contents); err != nil {
+			return nil, fmt.Errorf("failed to write fixed content to file %s: %w", file, err)
 		}
+
+		fixReport.AddFileFix(file, fixResult)
 	}
 
-	return nil
+	return fixReport, nil
 }
 
 // applyLinterFixes handles the application of fixes that require linter violation triggers.
@@ -214,10 +193,15 @@ func (f *Fixer) applyLinterFixes(
 		return fmt.Errorf("failed to list files: %w", err)
 	}
 
+	var versionsMap map[string]ast.RegoVersion
+	if f.versionsMap != nil {
+		versionsMap = f.versionsMap
+	}
+
 	for {
 		fixMadeInIteration := false
 
-		in, err := fp.ToInput()
+		in, err := fp.ToInput(versionsMap)
 		if err != nil {
 			return fmt.Errorf("failed to generate linter input: %w", err)
 		}
@@ -266,11 +250,8 @@ func (f *Fixer) applyLinterFixes(
 			fixResults, err := fixInstance.Fix(&fixCandidate, &fixes.RuntimeOptions{
 				BaseDir: util.FindClosestMatchingRoot(abs, f.registeredRoots),
 				Config:  config,
-				Locations: []ast.Location{
-					{
-						Row: violation.Location.Row,
-						Col: violation.Location.Column,
-					},
+				Locations: []report.Location{
+					violation.Location,
 				},
 			})
 			if err != nil {
