@@ -8,35 +8,39 @@ import (
 	"io/fs"
 	"log"
 	"os"
+	"path/filepath"
 	"slices"
 	"strings"
 	"sync"
+	"testing/fstest"
 
-	"github.com/gobwas/glob"
 	"gopkg.in/yaml.v3"
 
-	"github.com/open-policy-agent/opa/ast"
-	"github.com/open-policy-agent/opa/bundle"
-	"github.com/open-policy-agent/opa/metrics"
-	"github.com/open-policy-agent/opa/profiler"
-	"github.com/open-policy-agent/opa/rego"
-	"github.com/open-policy-agent/opa/topdown"
-	"github.com/open-policy-agent/opa/topdown/print"
+	"github.com/open-policy-agent/opa/v1/ast"
+	"github.com/open-policy-agent/opa/v1/bundle"
+	"github.com/open-policy-agent/opa/v1/metrics"
+	"github.com/open-policy-agent/opa/v1/profiler"
+	"github.com/open-policy-agent/opa/v1/rego"
+	"github.com/open-policy-agent/opa/v1/topdown"
+	"github.com/open-policy-agent/opa/v1/topdown/print"
+	outil "github.com/open-policy-agent/opa/v1/util"
 
 	rbundle "github.com/styrainc/regal/bundle"
 	rio "github.com/styrainc/regal/internal/io"
 	regalmetrics "github.com/styrainc/regal/internal/metrics"
-	"github.com/styrainc/regal/internal/parse"
 	"github.com/styrainc/regal/internal/util"
 	"github.com/styrainc/regal/pkg/builtins"
 	"github.com/styrainc/regal/pkg/config"
 	"github.com/styrainc/regal/pkg/report"
 	"github.com/styrainc/regal/pkg/rules"
+
+	"github.com/styrainc/roast/pkg/encoding"
+	"github.com/styrainc/roast/pkg/transform"
+	rutil "github.com/styrainc/roast/pkg/util"
 )
 
 // Linter stores data to use for linting.
 type Linter struct {
-	customRuleFS         fs.FS
 	printHook            print.Hook
 	metrics              metrics.Metrics
 	inputModules         *rules.Input
@@ -44,15 +48,15 @@ type Linter struct {
 	combinedCfg          *config.Config
 	dataBundle           *bundle.Bundle
 	pathPrefix           string
-	customRuleFSRootPath string
+	customRuleError      error
 	inputPaths           []string
 	ruleBundles          []*bundle.Bundle
-	customRulesPaths     []string
 	disable              []string
 	disableCategory      []string
 	enable               []string
 	enableCategory       []string
 	ignoreFiles          []string
+	customRuleModules    []*ast.Module
 	overriddenAggregates map[string][]report.Aggregate
 	useCollectQuery      bool
 	debugMode            bool
@@ -61,6 +65,7 @@ type Linter struct {
 	enableAll            bool
 	profiling            bool
 	instrumentation      bool
+	hasCustomRules       bool
 }
 
 //nolint:gochecknoglobals
@@ -103,7 +108,29 @@ func (l Linter) WithAddedBundle(b *bundle.Bundle) Linter {
 
 // WithCustomRules adds custom rules for evaluation, from the Rego (and data) files provided at paths.
 func (l Linter) WithCustomRules(paths []string) Linter {
-	l.customRulesPaths = paths
+	for _, path := range paths {
+		stat, err := os.Stat(path)
+		if err != nil {
+			l.customRuleError = fmt.Errorf("failed to stat custom rule file %s: %w", path, err)
+
+			return l
+		}
+
+		if stat.IsDir() {
+			l = l.WithCustomRulesFromFS(os.DirFS(path), ".")
+		} else {
+			contents, err := os.ReadFile(path)
+			if err != nil {
+				l.customRuleError = fmt.Errorf("failed to read custom rule file %s: %w", path, err)
+
+				return l
+			}
+
+			l = l.WithCustomRulesFromFS(fstest.MapFS{
+				filepath.Base(path): &fstest.MapFile{Data: contents},
+			}, ".")
+		}
+	}
 
 	return l
 }
@@ -111,8 +138,22 @@ func (l Linter) WithCustomRules(paths []string) Linter {
 // WithCustomRulesFromFS adds custom rules for evaluation from a filesystem implementing the fs.FS interface.
 // A root path within the filesystem must also be specified. Note, _test.rego files will be ignored.
 func (l Linter) WithCustomRulesFromFS(f fs.FS, rootPath string) Linter {
-	l.customRuleFS = f
-	l.customRuleFSRootPath = rootPath
+	if f == nil {
+		return l
+	}
+
+	l.hasCustomRules = true
+
+	modules, err := loadModulesFromCustomRuleFS(f, rootPath)
+	if err != nil {
+		l.customRuleError = err
+
+		return l
+	}
+
+	for _, m := range modules {
+		l.customRuleModules = append(l.customRuleModules, m)
+	}
 
 	return l
 }
@@ -248,8 +289,9 @@ func (l Linter) Lint(ctx context.Context) (report.Report, error) {
 
 	finalReport := report.Report{}
 
-	if len(l.inputPaths) == 0 && l.inputModules == nil && len(l.overriddenAggregates) == 0 {
-		return report.Report{}, errors.New("nothing provided to lint")
+	err := l.validate()
+	if err != nil {
+		return report.Report{}, fmt.Errorf("validation failed: %w", err)
 	}
 
 	conf, err := l.GetConfig()
@@ -257,19 +299,7 @@ func (l Linter) Lint(ctx context.Context) (report.Report, error) {
 		return report.Report{}, fmt.Errorf("failed to merge config: %w", err)
 	}
 
-	l.dataBundle = &bundle.Bundle{
-		Manifest: bundle.Manifest{
-			Roots:    &[]string{"internal"},
-			Metadata: map[string]any{"name": "internal"},
-		},
-		Data: map[string]any{
-			"internal": map[string]any{
-				"combined_config": config.ToMap(*conf),
-				"capabilities":    rio.ToMap(config.CapabilitiesForThisVersion()),
-				"path_prefix":     l.pathPrefix,
-			},
-		},
-	}
+	l.dataBundle = l.createDataBundle(*conf)
 
 	ignore := conf.Ignore.Files
 
@@ -287,7 +317,16 @@ func (l Linter) Lint(ctx context.Context) (report.Report, error) {
 	l.stopTimer(regalmetrics.RegalFilterIgnoredFiles)
 	l.startTimer(regalmetrics.RegalInputParse)
 
-	inputFromPaths, err := rules.InputFromPaths(filtered)
+	var versionsMap map[string]ast.RegoVersion
+
+	if l.pathPrefix != "" && !strings.HasPrefix(l.pathPrefix, "file://") {
+		versionsMap, err = config.AllRegoVersions(l.pathPrefix, conf)
+		if err != nil && l.debugMode {
+			log.Printf("failed to get configured Rego versions: %v", err)
+		}
+	}
+
+	inputFromPaths, err := rules.InputFromPaths(filtered, l.pathPrefix, versionsMap)
 	if err != nil {
 		return report.Report{}, fmt.Errorf("errors encountered when reading files to lint: %w", err)
 	}
@@ -318,13 +357,6 @@ func (l Linter) Lint(ctx context.Context) (report.Report, error) {
 		l.stopTimer(regalmetrics.RegalFilterIgnoredModules)
 	}
 
-	goReport, err := l.lintWithGoRules(ctx, input)
-	if err != nil {
-		return report.Report{}, fmt.Errorf("failed to lint using Go rules: %w", err)
-	}
-
-	finalReport.Violations = append(finalReport.Violations, goReport.Violations...)
-
 	regoArgs, err := l.prepareRegoArgs(lintQuery)
 	if err != nil {
 		return report.Report{}, fmt.Errorf("failed preparing query for linting: %w", err)
@@ -361,10 +393,6 @@ func (l Linter) Lint(ctx context.Context) (report.Report, error) {
 			allAggregates[k] = append(allAggregates[k], aggregates...)
 		}
 	} else if len(input.FileNames) > 1 {
-		for k, aggregates := range goReport.Aggregates {
-			allAggregates[k] = append(allAggregates[k], aggregates...)
-		}
-
 		for k, aggregates := range regoReport.Aggregates {
 			allAggregates[k] = append(allAggregates[k], aggregates...)
 		}
@@ -388,10 +416,6 @@ func (l Linter) Lint(ctx context.Context) (report.Report, error) {
 
 	if l.exportAggregates {
 		finalReport.Aggregates = make(map[string][]report.Aggregate)
-		for k, aggregates := range goReport.Aggregates {
-			finalReport.Aggregates[k] = append(finalReport.Aggregates[k], aggregates...)
-		}
-
 		for k, aggregates := range regoReport.Aggregates {
 			finalReport.Aggregates[k] = append(finalReport.Aggregates[k], aggregates...)
 		}
@@ -412,42 +436,102 @@ func (l Linter) Lint(ctx context.Context) (report.Report, error) {
 	return finalReport, nil
 }
 
+func (l Linter) validate() error {
+	if len(l.inputPaths) == 0 && l.inputModules == nil && len(l.overriddenAggregates) == 0 {
+		return errors.New("nothing provided to lint")
+	}
+
+	if l.customRuleError != nil {
+		return fmt.Errorf("failed to load custom rules: %w", l.customRuleError)
+	}
+
+	conf, err := l.GetConfig()
+	if err != nil {
+		return fmt.Errorf("failed to merge config: %w", err)
+	}
+
+	validCategories := rutil.NewSet[string]()
+	validRules := rutil.NewSet[string]()
+
+	// Add all built-in rules
+	for _, b := range l.ruleBundles {
+		for _, module := range b.Modules {
+			parts := util.UnquotedPath(module.Parsed.Package.Path)
+			// 1     2     3   4
+			// regal.rules.cat.rule
+			if len(parts) != 4 {
+				continue
+			}
+
+			validCategories.Add(parts[2])
+			validRules.Add(parts[3])
+		}
+	}
+
+	// Add any custom rules
+	for _, module := range l.customRuleModules {
+		parts := util.UnquotedPath(module.Package.Path)
+		// 1      2     3     4   5
+		// custom.regal.rules.cat.rule
+		if len(parts) != 5 {
+			continue
+		}
+
+		validCategories.Add(parts[3])
+		validRules.Add(parts[4])
+	}
+
+	configuredCategories := rutil.NewSet(outil.Keys(conf.Rules)...)
+	configuredRules := rutil.NewSet[string]()
+
+	for _, cat := range conf.Rules {
+		configuredRules.Add(outil.Keys(cat)...)
+	}
+
+	configuredRules.Add(l.enable...)
+	configuredRules.Add(l.disable...)
+	configuredCategories.Add(l.enableCategory...)
+	configuredCategories.Add(l.disableCategory...)
+
+	var invalidCategories []string
+
+	if diff := configuredCategories.Diff(validCategories); diff.Size() > 0 {
+		invalidCategories = diff.Items()
+	}
+
+	var invalidRules []string
+
+	if diff := configuredRules.Diff(validRules); diff.Size() > 0 {
+		invalidRules = diff.Items()
+	}
+
+	switch {
+	case len(invalidCategories) > 0 && len(invalidRules) > 0:
+		return fmt.Errorf("unknown categories: %v, unknown rules: %v", invalidCategories, invalidRules)
+	case len(invalidCategories) > 0:
+		return fmt.Errorf("unknown categories: %v", invalidCategories)
+	case len(invalidRules) > 0:
+		return fmt.Errorf("unknown rules: %v", invalidRules)
+	}
+
+	return nil
+}
+
 // DetermineEnabledRules returns the list of rules that are enabled based on
 // the supplied configuration. This makes use of the Rego and Go rule settings
 // to produce a single list of the rules that are to be run on this linter
 // instance.
 func (l Linter) DetermineEnabledRules(ctx context.Context) ([]string, error) {
-	enabledRules := make([]string, 0)
-
-	goRules, err := l.enabledGoRules()
-	if err != nil {
-		return nil, fmt.Errorf("failed to get enabled Go rules: %w", err)
-	}
-
-	for _, rule := range goRules {
-		enabledRules = append(enabledRules, rule.Name())
-	}
-
 	conf, err := l.GetConfig()
 	if err != nil {
 		return []string{}, fmt.Errorf("failed to merge config: %w", err)
 	}
 
-	l.dataBundle = &bundle.Bundle{
-		Manifest: bundle.Manifest{
-			Roots:    &[]string{"internal"},
-			Metadata: map[string]any{"name": "internal"},
-		},
-		Data: map[string]any{
-			"internal": map[string]any{
-				"combined_config": config.ToMap(*conf),
-				"capabilities":    rio.ToMap(config.CapabilitiesForThisVersion()),
-			},
-		},
-	}
+	l.dataBundle = l.createDataBundle(*conf)
 
 	queryStr := `[rule |
         data.regal.rules[cat][rule]
+		object.get(data.regal.rules[cat][rule], "notices", set()) == set()
         data.regal.config.for_rule(cat, rule).level != "ignore"
     ]`
 
@@ -472,6 +556,8 @@ func (l Linter) DetermineEnabledRules(ctx context.Context) ([]string, error) {
 		return nil, fmt.Errorf("expected list, got %T", rs[0].Expressions[0].Value)
 	}
 
+	enabledRules := make([]string, 0)
+
 	for _, item := range list {
 		rule, ok := item.(string)
 		if !ok {
@@ -496,18 +582,7 @@ func (l Linter) DetermineEnabledAggregateRules(ctx context.Context) ([]string, e
 		return []string{}, fmt.Errorf("failed to merge config: %w", err)
 	}
 
-	l.dataBundle = &bundle.Bundle{
-		Manifest: bundle.Manifest{
-			Roots:    &[]string{"internal"},
-			Metadata: map[string]any{"name": "internal"},
-		},
-		Data: map[string]any{
-			"internal": map[string]any{
-				"combined_config": config.ToMap(*conf),
-				"capabilities":    rio.ToMap(config.CapabilitiesForThisVersion()),
-			},
-		},
-	}
+	l.dataBundle = l.createDataBundle(*conf)
 
 	queryStr := `[rule |
         data.regal.rules[cat][rule].aggregate
@@ -549,135 +624,7 @@ func (l Linter) DetermineEnabledAggregateRules(ctx context.Context) ([]string, e
 	return enabledRules, nil
 }
 
-func (l Linter) lintWithGoRules(ctx context.Context, input rules.Input) (report.Report, error) {
-	l.startTimer(regalmetrics.RegalLintGo)
-	defer l.stopTimer(regalmetrics.RegalLintGo)
-
-	goRules, err := l.enabledGoRules()
-	if err != nil {
-		return report.Report{}, fmt.Errorf("failed to get configured Go rules: %w", err)
-	}
-
-	goReport := report.Report{}
-
-	for _, rule := range goRules {
-		inp, err := inputForRule(input, rule)
-		if err != nil {
-			return report.Report{}, fmt.Errorf("error encountered while filtering input files: %w", err)
-		}
-
-		result, err := rule.Run(ctx, inp)
-		if err != nil {
-			return report.Report{}, fmt.Errorf("error encountered in Go rule evaluation: %w", err)
-		}
-
-		goReport.Violations = append(goReport.Violations, result.Violations...)
-	}
-
-	return goReport, err
-}
-
-func inputForRule(input rules.Input, rule rules.Rule) (rules.Input, error) {
-	ignore := rule.Config().Ignore
-
-	var ignoreFiles []string
-
-	if ignore != nil {
-		ignoreFiles = ignore.Files
-	}
-
-	return filterInputFiles(input, ignoreFiles)
-}
-
-func filterInputFiles(input rules.Input, ignore []string) (rules.Input, error) {
-	if len(ignore) == 0 {
-		return input, nil
-	}
-
-	n := len(input.FileNames)
-	newInput := rules.Input{
-		FileNames:   make([]string, 0, n),
-		FileContent: make(map[string]string, n),
-		Modules:     make(map[string]*ast.Module, n),
-	}
-
-outer:
-	for _, f := range input.FileNames {
-		for _, pattern := range ignore {
-			if pattern == "" {
-				continue
-			}
-
-			excluded, err := excludeFile(pattern, f)
-			if err != nil {
-				return rules.Input{}, fmt.Errorf("failed to check for exclusion using pattern %s: %w", pattern, err)
-			}
-
-			if excluded {
-				continue outer
-			}
-		}
-
-		newInput.FileNames = append(newInput.FileNames, f)
-		newInput.FileContent[f] = input.FileContent[f]
-		newInput.Modules[f] = input.Modules[f]
-	}
-
-	return newInput, nil
-}
-
-// excludeFile imitates the pattern matching of .gitignore files
-// See `exclusion.rego` for details on the implementation.
-func excludeFile(pattern string, filename string) (bool, error) {
-	n := len(pattern)
-
-	// Internal slashes means path is relative to root, otherwise it can
-	// appear anywhere in the directory (--> **/)
-	if !strings.Contains(pattern[:n-1], "/") {
-		pattern = "**/" + pattern
-	}
-
-	// Leading slash?
-	pattern = strings.TrimPrefix(pattern, "/")
-
-	// Leading double-star?
-	var ps []string
-	if strings.HasPrefix(pattern, "**/") {
-		ps = []string{pattern, strings.TrimPrefix(pattern, "**/")}
-	} else {
-		ps = []string{pattern}
-	}
-
-	var ps1 []string
-
-	// trailing slash?
-	for _, p := range ps {
-		switch {
-		case strings.HasSuffix(p, "/"):
-			ps1 = append(ps1, p+"**")
-		case !strings.HasSuffix(p, "/") && !strings.HasSuffix(p, "**"):
-			ps1 = append(ps1, p, p+"/**")
-		default:
-			ps1 = append(ps1, p)
-		}
-	}
-
-	// Loop through patterns and return true on first match
-	for _, p := range ps1 {
-		g, err := glob.Compile(p, '/')
-		if err != nil {
-			return false, fmt.Errorf("failed to compile pattern %s: %w", p, err)
-		}
-
-		if g.Match(filename) {
-			return true, nil
-		}
-	}
-
-	return false, nil
-}
-
-func (l Linter) paramsToRulesConfig() map[string]any {
+func (l Linter) createDataBundle(conf config.Config) *bundle.Bundle {
 	params := map[string]any{
 		"disable_all":      l.disableAll,
 		"disable_category": util.NullToEmpty(l.disableCategory),
@@ -685,37 +632,33 @@ func (l Linter) paramsToRulesConfig() map[string]any {
 		"enable_all":       l.enableAll,
 		"enable_category":  util.NullToEmpty(l.enableCategory),
 		"enable":           util.NullToEmpty(l.enable),
+		"ignore_files":     util.NullToEmpty(l.ignoreFiles),
 	}
 
-	if l.ignoreFiles != nil {
-		params["ignore_files"] = l.ignoreFiles
-	}
-
-	return map[string]any{
-		"eval": map[string]any{
-			"params": params,
+	return &bundle.Bundle{
+		Manifest: bundle.Manifest{
+			Roots:    &[]string{"internal", "eval"},
+			Metadata: map[string]any{"name": "internal"},
+		},
+		Data: map[string]any{
+			"eval": map[string]any{
+				"params": params,
+			},
+			"internal": map[string]any{
+				"combined_config": config.ToMap(conf),
+				"capabilities":    rio.ToMap(config.CapabilitiesForThisVersion()),
+				"path_prefix":     l.pathPrefix,
+			},
 		},
 	}
 }
 
 func (l Linter) prepareRegoArgs(query ast.Body) ([]func(*rego.Rego), error) {
-	var regoArgs []func(*rego.Rego)
-
-	roots := []string{"eval"}
-
-	dataBundle := bundle.Bundle{
-		Data:     l.paramsToRulesConfig(),
-		Manifest: bundle.Manifest{Roots: &roots},
-	}
-
-	regoArgs = append(regoArgs,
+	regoArgs := append([]func(*rego.Rego){
 		rego.StoreReadAST(true),
 		rego.Metrics(l.metrics),
 		rego.ParsedQuery(query),
-		rego.ParsedBundle("regal_eval_params", &dataBundle),
-		rego.Function2(builtins.RegalParseModuleMeta, builtins.RegalParseModule),
-		rego.Function1(builtins.RegalLastMeta, builtins.RegalLast),
-	)
+	}, builtins.RegalBuiltinRegoFuncs...)
 
 	if l.debugMode && l.printHook == nil {
 		l.printHook = topdown.NewPrintHook(os.Stderr)
@@ -736,18 +679,13 @@ func (l Linter) prepareRegoArgs(query ast.Body) ([]func(*rego.Rego), error) {
 		regoArgs = append(regoArgs, rego.ParsedBundle("internal", l.dataBundle))
 	}
 
-	if l.customRulesPaths != nil {
-		regoArgs = append(regoArgs, rego.Load(l.customRulesPaths, rio.ExcludeTestFilter()))
-	}
-
-	if l.customRuleFS != nil && l.customRuleFSRootPath != "" {
-		files, err := loadModulesFromCustomRuleFS(l.customRuleFS, l.customRuleFSRootPath)
-		if err != nil {
-			return nil, fmt.Errorf("failed to load custom rules from FS: %w", err)
+	if l.hasCustomRules {
+		if l.customRuleError != nil {
+			return nil, fmt.Errorf("failed to load custom rules: %w", l.customRuleError)
 		}
 
-		for path, content := range files {
-			regoArgs = append(regoArgs, rego.Module(path, content))
+		for _, m := range l.customRuleModules {
+			regoArgs = append(regoArgs, rego.ParsedModule(m))
 		}
 	}
 
@@ -765,8 +703,8 @@ func (l Linter) prepareRegoArgs(query ast.Body) ([]func(*rego.Rego), error) {
 	return regoArgs, nil
 }
 
-func loadModulesFromCustomRuleFS(customRuleFS fs.FS, rootPath string) (map[string]string, error) {
-	files := make(map[string]string)
+func loadModulesFromCustomRuleFS(customRuleFS fs.FS, rootPath string) (map[string]*ast.Module, error) {
+	modules := make(map[string]*ast.Module)
 	filter := rio.ExcludeTestFilter()
 
 	err := fs.WalkDir(customRuleFS, rootPath, func(path string, d fs.DirEntry, err error) error {
@@ -798,7 +736,12 @@ func loadModulesFromCustomRuleFS(customRuleFS fs.FS, rootPath string) (map[strin
 			return fmt.Errorf("failed to read custom rule file: %w", err)
 		}
 
-		files[path] = string(bs)
+		m, err := ast.ParseModule(path, outil.ByteSliceToString(bs))
+		if err != nil {
+			return fmt.Errorf("failed to parse custom rule file %q: %w", path, err)
+		}
+
+		modules[path] = m
 
 		return nil
 	})
@@ -806,7 +749,7 @@ func loadModulesFromCustomRuleFS(customRuleFS fs.FS, rootPath string) (map[strin
 		return nil, fmt.Errorf("failed to walk custom rule FS: %w", err)
 	}
 
-	return files, nil
+	return modules, nil
 }
 
 func (l Linter) lintWithRegoRules(
@@ -845,26 +788,15 @@ func (l Linter) lintWithRegoRules(
 		go func(name string) {
 			defer wg.Done()
 
-			enhancedAST, err := parse.PrepareAST(name, input.FileContent[name], input.Modules[name])
+			inputValue, err := transform.ToAST(name, input.FileContent[name], input.Modules[name], operationCollect)
 			if err != nil {
-				errCh <- fmt.Errorf("failed preparing AST: %w", err)
+				errCh <- fmt.Errorf("failed to transform input value: %w", err)
 
 				return
 			}
 
-			regalInput, ok := enhancedAST["regal"].(map[string]any)
-			if ok {
-				operations := []string{"lint"}
-
-				if operationCollect {
-					operations = append(operations, "collect")
-				}
-
-				regalInput["operations"] = operations
-			}
-
 			evalArgs := []rego.EvalOption{
-				rego.EvalInput(enhancedAST),
+				rego.EvalParsedInput(inputValue),
 			}
 
 			if l.metrics != nil {
@@ -984,7 +916,12 @@ func (l Linter) lintWithRegoAggregateRules(
 		},
 	}
 
-	evalArgs := []rego.EvalOption{rego.EvalInput(input)}
+	inputValue, err := transform.ToOPAInputValue(input)
+	if err != nil {
+		return report.Report{}, fmt.Errorf("failed to transform input value: %w", err)
+	}
+
+	evalArgs := []rego.EvalOption{rego.EvalParsedInput(inputValue)}
 
 	if l.metrics != nil {
 		evalArgs = append(evalArgs, rego.EvalMetrics(l.metrics))
@@ -1017,14 +954,14 @@ func resultSetToReport(resultSet rego.ResultSet, aggregate bool) (report.Report,
 	if aggregate {
 		if binding, ok := resultSet[0].Bindings["lint"].(map[string]any); ok {
 			if aggregateBinding, ok := binding["aggregate"]; ok {
-				if err := rio.JSONRoundTrip(aggregateBinding, &r); err != nil {
+				if err := encoding.JSONRoundTrip(aggregateBinding, &r); err != nil {
 					return report.Report{}, fmt.Errorf("JSON rountrip failed for bindings: %v %w", binding, err)
 				}
 			}
 		}
 	} else {
 		if binding, ok := resultSet[0].Bindings["lint"]; ok {
-			if err := rio.JSONRoundTrip(binding, &r); err != nil {
+			if err := encoding.JSONRoundTrip(binding, &r); err != nil {
 				return report.Report{}, fmt.Errorf("JSON rountrip failed for bindings: %v %w", binding, err)
 			}
 		}
@@ -1057,7 +994,7 @@ func (l Linter) GetConfig() (*config.Config, error) {
 		}
 
 		log.Println("merged provided and user config:")
-		log.Println(string(bs))
+		log.Println(outil.ByteSliceToString(bs))
 	}
 
 	l.combinedCfg = &mergedConf
@@ -1065,70 +1002,6 @@ func (l Linter) GetConfig() (*config.Config, error) {
 	return l.combinedCfg, nil
 }
 
-func (l Linter) enabledGoRules() ([]rules.Rule, error) {
-	var enabledGoRules []rules.Rule
-
-	// enabling/disabling all rules takes precedence and entirely disregards configuration
-	// files, but still respects the enable/disable category or rule flags
-
-	if l.disableAll {
-		for _, rule := range rules.AllGoRules(config.Config{}) {
-			if slices.Contains(l.enableCategory, rule.Category()) || slices.Contains(l.enable, rule.Name()) {
-				enabledGoRules = append(enabledGoRules, rule)
-			}
-		}
-
-		return enabledGoRules, nil
-	}
-
-	if l.enableAll {
-		for _, rule := range rules.AllGoRules(config.Config{}) {
-			if !slices.Contains(l.disableCategory, rule.Category()) && !slices.Contains(l.disable, rule.Name()) {
-				enabledGoRules = append(enabledGoRules, rule)
-			}
-		}
-
-		return enabledGoRules, nil
-	}
-
-	conf, err := l.GetConfig()
-	if err != nil {
-		return nil, fmt.Errorf("failed to create merged config: %w", err)
-	}
-
-	for _, rule := range rules.AllGoRules(*conf) {
-		// disabling specific rule has the highest precedence
-		if slices.Contains(l.disable, rule.Name()) {
-			continue
-		}
-
-		// likewise for enabling specific rule
-		if slices.Contains(l.enable, rule.Name()) {
-			enabledGoRules = append(enabledGoRules, rule)
-
-			continue
-		}
-
-		// next highest precedence is disabling / enabling a category
-		if slices.Contains(l.disableCategory, rule.Category()) {
-			continue
-		}
-
-		if slices.Contains(l.enableCategory, rule.Category()) {
-			enabledGoRules = append(enabledGoRules, rule)
-
-			continue
-		}
-
-		// if none of the above applies, check the config for the rule
-		if rule.Config().Level != "ignore" {
-			enabledGoRules = append(enabledGoRules, rule)
-		}
-	}
-
-	return enabledGoRules, nil
-}
-
 func (l Linter) getBundleByName(name string) (*bundle.Bundle, error) {
 	if l.ruleBundles == nil {
 		return nil, errors.New("no bundles loaded")
