@@ -6,12 +6,13 @@ import (
 	"embed"
 	"path/filepath"
 	"slices"
+	"strings"
 	"testing"
 
 	"gopkg.in/yaml.v3"
 
-	"github.com/open-policy-agent/opa/ast"
-	"github.com/open-policy-agent/opa/topdown"
+	"github.com/open-policy-agent/opa/v1/ast"
+	"github.com/open-policy-agent/opa/v1/topdown"
 
 	"github.com/styrainc/regal/internal/parse"
 	"github.com/styrainc/regal/internal/test"
@@ -26,8 +27,6 @@ func TestLintWithDefaultBundle(t *testing.T) {
 
 	input := test.InputPolicy("p/p.rego", `package p
 
-import rego.v1
-
 # TODO: fix this
 camelCase if {
 	input.one == 1
@@ -47,7 +46,7 @@ camelCase if {
 		t.Errorf("expected first violation to be 'todo-comments', got %s", result.Violations[0].Title)
 	}
 
-	if result.Violations[0].Location.Row != 5 {
+	if result.Violations[0].Location.Row != 3 {
 		t.Errorf("expected first violation to be on line 3, got %d", result.Violations[0].Location.Row)
 	}
 
@@ -63,7 +62,7 @@ camelCase if {
 		t.Errorf("expected second violation to be 'prefer-snake-case', got %s", result.Violations[1].Title)
 	}
 
-	if result.Violations[1].Location.Row != 6 {
+	if result.Violations[1].Location.Row != 4 {
 		t.Errorf("expected second violation to be on line 4, got %d", result.Violations[1].Location.Row)
 	}
 
@@ -131,7 +130,7 @@ or := 1
 		{
 			name:          "baseline",
 			filename:      "p/p.rego",
-			expViolations: []string{"opa-fmt", "top-level-iteration", "rule-shadows-builtin"},
+			expViolations: []string{"top-level-iteration", "rule-shadows-builtin", "opa-fmt"},
 		},
 		{
 			name: "ignore rule",
@@ -202,8 +201,8 @@ or := 1
 				Rules: map[string]config.Category{},
 			},
 			filename:      "p/p.rego",
-			expViolations: []string{"opa-fmt", "top-level-iteration", "rule-shadows-builtin"},
-			expLevels:     []string{"error", "warning", "warning"},
+			expViolations: []string{"top-level-iteration", "rule-shadows-builtin", "opa-fmt"},
+			expLevels:     []string{"warning", "warning", "error"},
 		},
 		{
 			name: "rule level ignore files",
@@ -254,7 +253,7 @@ or := 1
 			},
 			filename: "file:///wow/foo/p.rego",
 			expViolations: []string{
-				"opa-fmt", "top-level-iteration", "rule-shadows-builtin", "directory-package-mismatch",
+				"top-level-iteration", "rule-shadows-builtin", "directory-package-mismatch", "opa-fmt",
 			},
 			rootDir: "file:///wow",
 		},
@@ -270,22 +269,16 @@ or := 1
 		t.Run(tc.name, func(t *testing.T) {
 			t.Parallel()
 
-			linter := NewLinter()
-
-			linter = linter.WithPathPrefix(tc.rootDir)
+			input := test.InputPolicy(tc.filename, policy)
+			linter := NewLinter().
+				WithPathPrefix(tc.rootDir).
+				WithIgnore(tc.ignoreFilesFlag).
+				WithInputModules(&input)
 
 			if tc.userConfig != nil {
 				linter = linter.WithUserConfig(*tc.userConfig)
 			}
 
-			if tc.ignoreFilesFlag != nil {
-				linter = linter.WithIgnore(tc.ignoreFilesFlag)
-			}
-
-			input := test.InputPolicy(tc.filename, policy)
-
-			linter = linter.WithInputModules(&input)
-
 			result := testutil.Must(linter.Lint(context.Background()))(t)
 
 			if len(result.Violations) != len(tc.expViolations) {
@@ -313,54 +306,6 @@ or := 1
 	}
 }
 
-func TestLintWithGoRule(t *testing.T) {
-	t.Parallel()
-
-	input := test.InputPolicy("p/p.rego", `package p
-		import rego.v1
-
- 		x := true
-	`)
-
-	linter := NewLinter().
-		WithEnableAll(true).
-		WithInputModules(&input)
-
-	result := testutil.Must(linter.Lint(context.Background()))(t)
-
-	if len(result.Violations) != 1 {
-		t.Fatalf("expected 1 violation, got %d", len(result.Violations))
-	}
-
-	if result.Violations[0].Title != "opa-fmt" {
-		t.Errorf("expected first violation to be 'opa-fmt', got %s", result.Violations[0].Title)
-	}
-}
-
-func TestLintWithUserConfigGoRuleIgnore(t *testing.T) {
-	t.Parallel()
-
-	userConfig := config.Config{Rules: map[string]config.Category{
-		"style": {"opa-fmt": config.Rule{Level: "ignore"}},
-	}}
-
-	input := test.InputPolicy("p/p.rego", `package p
-		import rego.v1
-
-	 	x := true
-	`)
-
-	linter := NewLinter().
-		WithUserConfig(userConfig).
-		WithInputModules(&input)
-
-	result := testutil.Must(linter.Lint(context.Background()))(t)
-
-	if len(result.Violations) != 0 {
-		t.Fatalf("expected no violation, got %d", len(result.Violations))
-	}
-}
-
 func TestLintWithCustomRule(t *testing.T) {
 	t.Parallel()
 
@@ -381,6 +326,26 @@ func TestLintWithCustomRule(t *testing.T) {
 	}
 }
 
+func TestLintWithErrorInEnable(t *testing.T) {
+	t.Parallel()
+
+	input := test.InputPolicy("p/p.rego", "package p")
+
+	linter := NewLinter().
+		WithCustomRules([]string{filepath.Join("testdata", "custom.rego")}).
+		WithEnabledRules("foo").
+		WithInputModules(&input)
+
+	_, err := linter.Lint(context.Background())
+	if err == nil {
+		t.Fatal("expected error, got nil")
+	}
+
+	if exp, got := "unknown rules: [foo]", err.Error(); !strings.Contains(got, exp) {
+		t.Fatalf("expected error to contain %q, got %q", exp, got)
+	}
+}
+
 //go:embed testdata/*
 var testLintWithCustomEmbeddedRulesFS embed.FS
 
@@ -452,7 +417,7 @@ func TestLintMergedConfigInheritsLevelFromProvided(t *testing.T) {
 	fileLength := mergedConfig.Rules["style"]["file-length"].Extra["max-file-length"]
 
 	// Ensure the extra attributes are still there.
-	if fileLength.(int) != 1 { // nolint: forcetypeassert
+	if fileLength.(int) != 1 { //nolint: forcetypeassert
 		t.Errorf("expected max-file-length to be 1, got %d %T", fileLength, fileLength)
 	}
 }
@@ -761,16 +726,57 @@ import data.unresolved`,
 	}
 }
 
+// 1809032500 ns/op	3227205712 B/op	63995081 allocs/op
+// 1692214541 ns/op	3089440208 B/op	60822121 allocs/op main
+// 1654144083 ns/op	3044255536 B/op	60234057 allocs/op opa-fmt-remodel
+// ...
 func BenchmarkRegalLintingItself(b *testing.B) {
-	linter := NewLinter().WithInputPaths([]string{"../../bundle"}).WithEnableAll(true)
+	linter := NewLinter().
+		WithInputPaths([]string{"../../bundle"}).
+		WithEnableAll(true)
 
 	b.ResetTimer()
 	b.ReportAllocs()
 
-	for i := 0; i < b.N; i++ {
-		_, err := linter.Lint(context.Background())
+	var err error
+
+	var rep report.Report
+
+	for range b.N {
+		rep, err = linter.Lint(context.Background())
 		if err != nil {
 			b.Fatal(err)
 		}
 	}
+
+	if len(rep.Violations) != 0 {
+		_ = rep.Violations
+	}
+}
+
+// 268203979 ns/op	501748262 B/op	 9724107 allocs/op
+// 258819136 ns/op	497016108 B/op	 9632445 allocs/op
+// ...
+func BenchmarkRegalNoEnabledRules(b *testing.B) {
+	linter := NewLinter().
+		WithInputPaths([]string{"../../bundle"}).
+		WithDisableAll(true)
+
+	b.ResetTimer()
+	b.ReportAllocs()
+
+	var err error
+
+	var rep report.Report
+
+	for range b.N {
+		rep, err = linter.Lint(context.Background())
+		if err != nil {
+			b.Fatal(err)
+		}
+	}
+
+	if len(rep.Violations) != 0 {
+		_ = rep.Violations
+	}
 }
